from enum import Enum
from typing import Any, Optional

from flet.core.control import Control
from flet.core.ref import Ref
from flet.utils import deprecated


class PermissionStatus(Enum):
    GRANTED = "granted"
    DENIED = "denied"
    PERMANENTLY_DENIED = "permanentlyDenied"
    LIMITED = "limited"
    PROVISIONAL = "provisional"
    RESTRICTED = "restricted"


class PermissionType(Enum):
    ACCESS_MEDIA_LOCATION = "accessMediaLocation"
    ACCESS_NOTIFICATION_POLICY = "accessNotificationPolicy"
    ACTIVITY_RECOGNITION = "activityRecognition"
    APP_TRACKING_TRANSPARENCY = "appTrackingTransparency"
    ASSISTANT = "assistant"
    AUDIO = "audio"
    BACKGROUND_REFRESH = "backgroundRefresh"
    BLUETOOTH = "bluetooth"
    BLUETOOTH_ADVERTISE = "bluetoothAdvertise"
    BLUETOOTH_CONNECT = "bluetoothConnect"
    BLUETOOTH_SCAN = "bluetoothScan"
    CALENDAR_FULL_ACCESS = "calendarFullAccess"
    CALENDAR_WRITE_ONLY = "calendarWriteOnly"
    CAMERA = "camera"
    CONTACTS = "contacts"
    CRITICAL_ALERTS = "criticalAlerts"
    IGNORE_BATTERY_OPTIMIZATIONS = "ignoreBatteryOptimizations"
    LOCATION = "location"
    LOCATION_ALWAYS = "locationAlways"
    LOCATION_WHEN_IN_USE = "locationWhenInUse"
    MANAGE_EXTERNAL_STORAGE = "manageExternalStorage"
    MEDIA_LIBRARY = "mediaLibrary"
    MICROPHONE = "microphone"
    NEARBY_WIFI_DEVICES = "nearbyWifiDevices"
    NOTIFICATION = "notification"
    PHONE = "phone"
    PHOTOS = "photos"
    PHOTOS_ADD_ONLY = "photosAddOnly"
    REMINDERS = "reminders"
    REQUEST_INSTALL_PACKAGES = "requestInstallPackages"
    SCHEDULE_EXACT_ALARM = "scheduleExactAlarm"
    SENSORS = "sensors"
    SENSORS_ALWAYS = "sensorsAlways"
    SMS = "sms"
    SPEECH = "speech"
    STORAGE = "storage"
    SYSTEM_ALERT_WINDOW = "systemAlertWindow"
    UNKNOWN = "unknown"
    VIDEOS = "videos"


@deprecated(
    reason="PermissionHandler control has been moved to a separate Python package: https://pypi.org/project/flet-permission-handler. "
    + "Read more about this change in Flet blog: https://flet.dev/blog/flet-v-0-26-release-announcement",
    version="0.26.0",
    delete_version="0.29.0",
)
class PermissionHandler(Control):
    """
    A control that allows you check and request permission from your device.
    This control is non-visual and should be added to `page.overlay` list.

    -----

    Online docs: https://flet.dev/docs/controls/permissionhandler
    """

    def __init__(
        self,
        # Control
        #
        ref: Optional[Ref] = None,
        data: Any = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            data=data,
        )

    def _get_control_name(self):
        return "permission_handler"

    def check_permission(
        self, of: PermissionType, wait_timeout: Optional[float] = 25
    ) -> Optional[PermissionStatus]:
        out = self.invoke_method(
            "check_permission",
            {"of": of.value if isinstance(of, PermissionType) else of},
            wait_for_result=True,
            wait_timeout=wait_timeout,
        )
        return PermissionStatus(out) if out is not None else None

    async def check_permission_async(
        self, of: PermissionType, wait_timeout: Optional[float] = 25
    ) -> Optional[PermissionStatus]:
        out = await self.invoke_method_async(
            "check_permission",
            {"of": of.value if isinstance(of, PermissionType) else of},
            wait_for_result=True,
            wait_timeout=wait_timeout,
        )
        return PermissionStatus(out) if out is not None else None

    def request_permission(
        self, of: PermissionType, wait_timeout: Optional[float] = 25
    ) -> Optional[PermissionStatus]:
        out = self.invoke_method(
            "request_permission",
            {"of": of.value if isinstance(of, PermissionType) else of},
            wait_for_result=True,
            wait_timeout=wait_timeout,
        )
        return PermissionStatus(out) if out is not None else None

    async def request_permission_async(
        self, of: PermissionType, wait_timeout: Optional[float] = 25
    ) -> Optional[PermissionStatus]:
        out = await self.invoke_method_async(
            "request_permission",
            {"of": of.value if isinstance(of, PermissionType) else of},
            wait_for_result=True,
            wait_timeout=wait_timeout,
        )
        return PermissionStatus(out) if out is not None else None

    def open_app_settings(self, wait_timeout: Optional[float] = 10) -> bool:
        opened = self.invoke_method(
            "open_app_settings",
            wait_for_result=True,
            wait_timeout=wait_timeout,
        )
        return opened == "true"

    async def open_app_settings_async(self, wait_timeout: Optional[float] = 10) -> bool:
        opened = await self.invoke_method_async(
            "open_app_settings",
            wait_for_result=True,
            wait_timeout=wait_timeout,
        )
        return opened == "true"



from enum import Enum
from typing import Any, List, Optional, Sequence

from flet.core.buttons import OutlinedBorder
from flet.core.control import Control
from flet.core.ref import Ref
from flet.core.types import (
    ColorEnums,
    ColorValue,
    IconEnums,
    IconValueOrControl,
    OptionalControlEventCallable,
    OptionalNumber,
    PaddingValue,
)


class NavigationDrawerDestination(Control):
    """Displays an icon with a label, for use in NavigationDrawer destinations."""

    def __init__(
        self,
        label: Optional[str] = None,
        icon: Optional[IconValueOrControl] = None,
        selected_icon: Optional[IconValueOrControl] = None,
        bgcolor: Optional[ColorValue] = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        data: Any = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            disabled=disabled,
            data=data,
            visible=visible,
        )
        self.label = label
        self.bgcolor = bgcolor
        self.icon = icon
        self.selected_icon = selected_icon

    def _get_control_name(self):
        return "navigationdrawerdestination"

    def _get_children(self):
        children = []
        if isinstance(self.__icon, Control):
            self.__icon._set_attr_internal("n", "icon")
            children.append(self.__icon)
        if isinstance(self.__selected_icon, Control):
            self.__selected_icon._set_attr_internal("n", "selected_icon")
            children.append(self.__selected_icon)
        return children

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgColor", value, ColorEnums)

    # icon
    @property
    def icon(self) -> Optional[IconValueOrControl]:
        return self.__icon

    @icon.setter
    def icon(self, value: Optional[IconValueOrControl]):
        self.__icon = value
        if not isinstance(value, Control):
            self._set_enum_attr("icon", value, IconEnums)

    # selected_icon
    @property
    def selected_icon(self) -> Optional[IconValueOrControl]:
        return self.__selected_icon

    @selected_icon.setter
    def selected_icon(self, value: Optional[IconValueOrControl]):
        self.__selected_icon = value
        if not isinstance(value, Control):
            self._set_enum_attr("selectedIcon", value, IconEnums)

    # label
    @property
    def label(self) -> Optional[str]:
        return self._get_attr("label")

    @label.setter
    def label(self, value: Optional[str]):
        self._set_attr("label", value)


class NavigationDrawerPosition(Enum):
    START = "start"
    END = "end"


class NavigationDrawer(Control):
    """
    Material Design Navigation Drawer component.

    Navigation Drawer is a panel slides in horizontally from the left or right edge of a page to show primary destinations in an app.

    Example:

    ```
    import flet as ft


    def main(page: ft.Page):
        page.horizontal_alignment = ft.CrossAxisAlignment.CENTER

        def handle_dismissal(e):
            page.add(ft.Text("Drawer dismissed"))

        def handle_change(e):
            page.add(ft.Text(f"Selected Index changed: {e.selected_index}"))
            # page.close(drawer)

        drawer = ft.NavigationDrawer(
            on_dismiss=handle_dismissal,
            on_change=handle_change,
            controls=[
                ft.Container(height=12),
                ft.NavigationDrawerDestination(
                    label="Item 1",
                    icon=ft.icons.DOOR_BACK_DOOR_OUTLINED,
                    selected_icon_content=ft.Icon(ft.icons.DOOR_BACK_DOOR),
                ),
                ft.Divider(thickness=2),
                ft.NavigationDrawerDestination(
                    icon_content=ft.Icon(ft.icons.MAIL_OUTLINED),
                    label="Item 2",
                    selected_icon=ft.icons.MAIL,
                ),
                ft.NavigationDrawerDestination(
                    icon_content=ft.Icon(ft.icons.PHONE_OUTLINED),
                    label="Item 3",
                    selected_icon=ft.icons.PHONE,
                ),
            ],
        )

        page.add(ft.ElevatedButton("Show drawer", on_click=lambda e: page.open(drawer)))


    ft.app(main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/navigationdrawer
    """

    def __init__(
        self,
        controls: Optional[Sequence[Control]] = None,
        open: bool = False,
        selected_index: Optional[int] = None,
        bgcolor: Optional[ColorValue] = None,
        elevation: OptionalNumber = None,
        indicator_color: Optional[ColorValue] = None,
        indicator_shape: Optional[OutlinedBorder] = None,
        shadow_color: Optional[ColorValue] = None,
        surface_tint_color: Optional[ColorValue] = None,
        tile_padding: Optional[PaddingValue] = None,
        position: Optional[NavigationDrawerPosition] = None,
        on_change: OptionalControlEventCallable = None,
        on_dismiss: OptionalControlEventCallable = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
        data: Any = None,
        rtl: Optional[bool] = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            visible=visible,
            disabled=disabled,
            data=data,
            rtl=rtl,
        )

        self.open = open
        self.controls = controls
        self.selected_index = selected_index
        self.bgcolor = bgcolor
        self.elevation = elevation
        self.indicator_color = indicator_color
        self.indicator_shape = indicator_shape
        self.shadow_color = shadow_color
        self.surface_tint_color = surface_tint_color
        self.tile_padding = tile_padding
        self.position = position
        self.on_change = on_change
        self.on_dismiss = on_dismiss

    def _get_control_name(self):
        return "navigationdrawer"

    def before_update(self):
        super().before_update()
        self._set_attr_json("indicatorShape", self.__indicator_shape)
        self._set_attr_json("tilePadding", self.__tile_padding)

    def _get_children(self):
        return self.__controls

    # open
    @property
    def open(self) -> bool:
        return self._get_attr("open", data_type="bool", def_value=False)

    @open.setter
    def open(self, value: Optional[bool]):
        self._set_attr("open", value)

    # controls
    @property
    def controls(self) -> Optional[List[Control]]:
        return self.__controls

    @controls.setter
    def controls(self, value: Optional[Sequence[Control]]):
        self.__controls = list(value) if value is not None else []

    # selected_index
    @property
    def selected_index(self) -> int:
        return self._get_attr("selectedIndex", data_type="int", def_value=0)

    @selected_index.setter
    def selected_index(self, value: Optional[int]):
        self._set_attr("selectedIndex", value)

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # position
    @property
    def position(self) -> Optional[NavigationDrawerPosition]:
        return self.__position

    @position.setter
    def position(self, value: Optional[NavigationDrawerPosition]):
        self.__position = value or NavigationDrawerPosition.START

    # elevation
    @property
    def elevation(self) -> OptionalNumber:
        return self._get_attr("elevation")

    @elevation.setter
    def elevation(self, value: OptionalNumber):
        self._set_attr("elevation", value)

    # indicator_color
    @property
    def indicator_color(self) -> Optional[ColorValue]:
        return self.__indicator_color

    @indicator_color.setter
    def indicator_color(self, value: Optional[ColorValue]):
        self.__indicator_color = value
        self._set_enum_attr("indicatorColor", value, ColorEnums)

    # indicator_shape
    @property
    def indicator_shape(self) -> Optional[OutlinedBorder]:
        return self.__indicator_shape

    @indicator_shape.setter
    def indicator_shape(self, value: Optional[OutlinedBorder]):
        self.__indicator_shape = value

    # shadow_color
    @property
    def shadow_color(self) -> Optional[ColorValue]:
        return self.__shadow_color

    @shadow_color.setter
    def shadow_color(self, value: Optional[ColorValue]):
        self.__shadow_color = value
        self._set_enum_attr("shadowColor", value, ColorEnums)

    # surface_tint_color
    @property
    def surface_tint_color(self) -> Optional[ColorValue]:
        return self.__surface_tint_color

    @surface_tint_color.setter
    def surface_tint_color(self, value: Optional[ColorValue]):
        self.__surface_tint_color = value
        self._set_enum_attr("surfaceTintColor", value, ColorEnums)

    # tile_padding
    @property
    def tile_padding(self) -> Optional[PaddingValue]:
        return self.__tile_padding

    @tile_padding.setter
    def tile_padding(self, value: Optional[PaddingValue]):
        self.__tile_padding = value

    # on_change
    @property
    def on_change(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change")

    @on_change.setter
    def on_change(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change", handler)

    # on_dismiss
    @property
    def on_dismiss(self) -> OptionalControlEventCallable:
        return self._get_event_handler("dismiss")

    @on_dismiss.setter
    def on_dismiss(self, handler: OptionalControlEventCallable):
        self._add_event_handler("dismiss", handler)



from enum import Enum
from typing import Any, Optional, Union

from flet.core.adaptive_control import AdaptiveControl
from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    ControlStateValue,
    MouseCursor,
    OffsetValue,
    OptionalControlEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
    PaddingValue,
)
from flet.utils.deprecated import deprecated_property


class SliderInteraction(Enum):
    TAP_AND_SLIDE = "tapAndSlide"
    TAP_ONLY = "tapOnly"
    SLIDE_ONLY = "slideOnly"
    SLIDE_THUMB = "slideThumb"


class Slider(ConstrainedControl, AdaptiveControl):
    """
    A slider provides a visual indication of adjustable content, as well as the current setting in the total range of content.

    Use a slider when you want people to set defined values (such as volume or brightness), or when people would benefit from instant feedback on the effect of setting changes.

    Example:
    ```
    import flet as ft

    def main(page):
        page.add(
            ft.Text("Slider with value:"),
            ft.Slider(value=0.3),
            ft.Text("Slider with a custom range and label:"),
            ft.Slider(min=0, max=100, divisions=10, label="{value}%"))

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/slider
    """

    def __init__(
        self,
        value: OptionalNumber = None,
        label: Optional[str] = None,
        min: OptionalNumber = None,
        max: OptionalNumber = None,
        divisions: Optional[int] = None,
        round: Optional[int] = None,
        autofocus: Optional[bool] = None,
        active_color: Optional[ColorValue] = None,
        inactive_color: Optional[ColorValue] = None,
        thumb_color: Optional[ColorValue] = None,
        interaction: Optional[SliderInteraction] = None,
        secondary_active_color: Optional[ColorValue] = None,
        overlay_color: ControlStateValue[ColorValue] = None,
        secondary_track_value: OptionalNumber = None,
        mouse_cursor: Optional[MouseCursor] = None,
        padding: Optional[PaddingValue] = None,
        year_2023: Optional[bool] = None,
        on_change: OptionalControlEventCallable = None,
        on_change_start: OptionalControlEventCallable = None,
        on_change_end: OptionalControlEventCallable = None,
        on_focus: OptionalControlEventCallable = None,
        on_blur: OptionalControlEventCallable = None,
        #
        # ConstrainedControl and AdaptiveControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
        adaptive: Optional[bool] = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        AdaptiveControl.__init__(self, adaptive=adaptive)

        self.value = value
        self.label = label
        self.min = min
        self.max = max
        self.divisions = divisions
        self.round = round
        self.autofocus = autofocus
        self.active_color = active_color
        self.inactive_color = inactive_color
        self.thumb_color = thumb_color
        self.interaction = interaction
        self.overlay_color = overlay_color
        self.on_change = on_change
        self.on_change_start = on_change_start
        self.on_change_end = on_change_end
        self.on_focus = on_focus
        self.on_blur = on_blur
        self.secondary_active_color = secondary_active_color
        self.secondary_track_value = secondary_track_value
        self.mouse_cursor = mouse_cursor
        self.padding = padding
        self.year_2023 = year_2023

    def _get_control_name(self):
        return "slider"

    def before_update(self):
        super().before_update()
        assert (
            self.min is None or self.max is None or self.min <= self.max
        ), "min must be less than or equal to max"
        assert (
            self.min is None or self.value is None or (self.value >= self.min)
        ), "value must be greater than or equal to min"
        assert (
            self.max is None or self.value is None or (self.value <= self.max)
        ), "value must be less than or equal to max"
        self._set_attr_json("overlayColor", self.__overlay_color, wrap_attr_dict=True)
        self._set_attr_json("padding", self.__padding)

    # value
    @property
    def value(self) -> float:
        return self._get_attr("value", data_type="float", def_value=self.min or 0.0)

    @value.setter
    def value(self, value: OptionalNumber):
        self._set_attr("value", value)

    # label
    @property
    def label(self) -> Optional[str]:
        return self._get_attr("label")

    @label.setter
    def label(self, value: Optional[str]):
        self._set_attr("label", value)

    # year_2023
    @property
    def year_2023(self) -> Optional[bool]:
        return self._get_attr("year2023", data_type="bool", def_value=True)

    @year_2023.setter
    def year_2023(self, value: Optional[bool]):
        self._set_attr("year2023", value)
        if value is not None:
            deprecated_property(
                name="year_2023",
                version="0.27.0",
                delete_version=None,  # not known for now
                reason="Set this flag to False to opt into the 2024 Slider appearance. In the future, this flag will default to False.",
            )

    # padding
    @property
    def padding(self) -> Optional[PaddingValue]:
        return self.__padding

    @padding.setter
    def padding(self, value: Optional[PaddingValue]):
        self.__padding = value

    # interaction
    @property
    def interaction(self) -> Optional[SliderInteraction]:
        return self.__interaction

    @interaction.setter
    def interaction(self, value: Optional[SliderInteraction]):
        self.__interaction = value
        self._set_enum_attr("interaction", value, SliderInteraction)

    # min
    @property
    def min(self) -> float:
        return self._get_attr("min", data_type="float", def_value=0.0)

    @min.setter
    def min(self, value: OptionalNumber):
        self._set_attr("min", value)

    # secondary_track_value
    @property
    def secondary_track_value(self) -> OptionalNumber:
        return self._get_attr("secondaryTrackValue", data_type="float")

    @secondary_track_value.setter
    def secondary_track_value(self, value: OptionalNumber):
        self._set_attr("secondaryTrackValue", value)

    # secondary_active_color
    @property
    def secondary_active_color(self) -> Optional[ColorValue]:
        return self.__secondary_active_color

    @secondary_active_color.setter
    def secondary_active_color(self, value: Optional[ColorValue]):
        self.__secondary_active_color = value
        self._set_enum_attr("secondaryActiveColor", value, ColorEnums)

    # mouse_cursor
    @property
    def mouse_cursor(self) -> Optional[MouseCursor]:
        return self._get_attr("mouseCursor")

    @mouse_cursor.setter
    def mouse_cursor(self, value: Optional[MouseCursor]):
        self._set_enum_attr("mouseCursor", value, MouseCursor)

    # max
    @property
    def max(self) -> float:
        return self._get_attr("max", data_type="float", def_value=1.0)

    @max.setter
    def max(self, value: OptionalNumber):
        self._set_attr("max", value)

    # divisions
    @property
    def divisions(self) -> Optional[int]:
        return self._get_attr("divisions")

    @divisions.setter
    def divisions(self, value: Optional[int]):
        self._set_attr("divisions", value)

    # round
    @property
    def round(self) -> int:
        return self._get_attr("round", data_type="int", def_value=0)

    @round.setter
    def round(self, value: Optional[int]):
        self._set_attr("round", value)

    # overlay_color
    @property
    def overlay_color(self) -> ControlStateValue[str]:
        return self.__overlay_color

    @overlay_color.setter
    def overlay_color(self, value: ControlStateValue[str]):
        self.__overlay_color = value

    # autofocus
    @property
    def autofocus(self) -> bool:
        return self._get_attr("autofocus", data_type="bool", def_value=False)

    @autofocus.setter
    def autofocus(self, value: Optional[bool]):
        self._set_attr("autofocus", value)

    # active_color
    @property
    def active_color(self) -> Optional[ColorValue]:
        return self.__active_color

    @active_color.setter
    def active_color(self, value: Optional[ColorValue]):
        self.__active_color = value
        self._set_enum_attr("activeColor", value, ColorEnums)

    # inactive_color
    @property
    def inactive_color(self) -> Optional[ColorValue]:
        return self.__inactive_color

    @inactive_color.setter
    def inactive_color(self, value: Optional[ColorValue]):
        self.__inactive_color = value
        self._set_enum_attr("inactiveColor", value, ColorEnums)

    # thumb_color
    @property
    def thumb_color(self) -> Optional[ColorValue]:
        return self.__thumb_color

    @thumb_color.setter
    def thumb_color(self, value: Optional[ColorValue]):
        self.__thumb_color = value
        self._set_enum_attr("thumbColor", value, ColorEnums)

    # on_change
    @property
    def on_change(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change")

    @on_change.setter
    def on_change(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change", handler)

    # on_change_start
    @property
    def on_change_start(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change_start")

    @on_change_start.setter
    def on_change_start(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change_start", handler)

    # on_change_end
    @property
    def on_change_end(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change_end")

    @on_change_end.setter
    def on_change_end(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change_end", handler)

    # on_focus
    @property
    def on_focus(self) -> OptionalControlEventCallable:
        return self._get_event_handler("focus")

    @on_focus.setter
    def on_focus(self, handler: OptionalControlEventCallable):
        self._add_event_handler("focus", handler)

    # on_blur
    @property
    def on_blur(self) -> OptionalControlEventCallable:
        return self._get_event_handler("blur")

    @on_blur.setter
    def on_blur(self, handler: OptionalControlEventCallable):
        self._add_event_handler("blur", handler)



from flet.core.ads.banner import BannerAd
from flet.core.ads.interstitial import InterstitialAd

"""
from flet.core.ads.native import (
    NativeAd,
    NativeAdTemplateStyle,
    NativeAdTemplateTextStyle,
    NativeTemplateFontStyle,
    NativeAdTemplateType,
)
"""



from dataclasses import dataclass
from enum import Enum
from typing import Any, List, Optional, Union

from flet.core.control import Control, OptionalNumber
from flet.core.map.map import MapLatitudeLongitude
from flet.core.map.map_layer import MapLayer
from flet.core.ref import Ref
from flet.core.types import ColorEnums, ColorValue, StrokeCap, StrokeJoin


class PatternFit(Enum):
    SCALE_DOWN = "scaleDown"
    SCALE_UP = "scaleUp"
    APPEND_DOT = "appendDot"
    EXTEND_FINAL_DASH = "extendFinalDash"


@dataclass
class StrokePattern:
    pass


@dataclass
class SolidStrokePattern(StrokePattern):
    def __post_init__(self):
        self.type = "solid"


@dataclass
class DashedStrokePattern(StrokePattern):
    segments: Optional[List[Union[float, int]]] = None
    pattern_fit: Optional[PatternFit] = PatternFit.SCALE_UP

    def __post_init__(self):
        self.type = "dashed"


@dataclass
class DottedStrokePattern(StrokePattern):
    spacing_factor: OptionalNumber = None
    pattern_fit: Optional[PatternFit] = PatternFit.SCALE_UP

    def __post_init__(self):
        self.type = "dotted"


class PolylineMarker(Control):
    """
    A marker for the PolylineLayer.

    -----

    Online docs: https://flet.dev/docs/controls/mappolylinemarker
    """

    def __init__(
        self,
        coordinates: List[MapLatitudeLongitude],
        colors_stop: Optional[List[Union[float, int]]] = None,
        gradient_colors: Optional[List[str]] = None,
        border_color: Optional[ColorValue] = None,
        color: Optional[ColorValue] = None,
        stroke_width: OptionalNumber = None,
        border_stroke_width: OptionalNumber = None,
        use_stroke_width_in_meter: Optional[bool] = None,
        stroke_pattern: Optional[StrokePattern] = None,
        stroke_cap: Optional[StrokeCap] = None,
        stroke_join: Optional[StrokeJoin] = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )

        self.coordinates = coordinates
        self.border_color = border_color
        self.color = color
        self.border_stroke_width = border_stroke_width
        self.stroke_width = stroke_width
        self.stroke_cap = stroke_cap
        self.stroke_join = stroke_join
        self.colors_stop = colors_stop
        self.gradient_colors = gradient_colors
        self.use_stroke_width_in_meter = use_stroke_width_in_meter
        self.stroke_pattern = stroke_pattern

    def _get_control_name(self):
        return "map_polyline_marker"

    def before_update(self):
        super().before_update()
        self._set_attr_json("coordinates", self.__coordinates)
        if isinstance(self.__colors_stop, list):
            self._set_attr_json("colorsStop", self.__colors_stop)
        if isinstance(self.__gradient_colors, list):
            self._set_attr_json("gradientColors", self.__gradient_colors)
        self._set_attr_json("strokePattern", self.__stroke_pattern)

    # stroke_cap
    @property
    def stroke_cap(self) -> Optional[StrokeCap]:
        return self.__stroke_cap

    @stroke_cap.setter
    def stroke_cap(self, value: Optional[StrokeCap]):
        self.__stroke_cap = value
        self._set_enum_attr("strokeCap", value, StrokeCap)

    # gradient_colors
    @property
    def gradient_colors(self) -> Optional[List[str]]:
        return self.__gradient_colors

    @gradient_colors.setter
    def gradient_colors(self, value: Optional[List[str]]):
        self.__gradient_colors = value

    # stroke_pattern
    @property
    def stroke_pattern(self) -> Optional[StrokePattern]:
        return self.__stroke_pattern

    @stroke_pattern.setter
    def stroke_pattern(self, value: Optional[StrokePattern]):
        self.__stroke_pattern = value

    # colors_stop
    @property
    def colors_stop(self) -> Optional[List[Union[float, int]]]:
        return self.__colors_stop

    @colors_stop.setter
    def colors_stop(self, value: Optional[List[Union[float, int]]]):
        self.__colors_stop = value

    # stroke_join
    @property
    def stroke_join(self) -> Optional[StrokeJoin]:
        return self.__stroke_join

    @stroke_join.setter
    def stroke_join(self, value: Optional[StrokeJoin]):
        self.__stroke_join = value
        self._set_enum_attr("strokeJoin", value, StrokeJoin)

    # use_stroke_width_in_meter
    @property
    def use_stroke_width_in_meter(self) -> bool:
        return self._get_attr(
            "useStrokeWidthInMeter", data_type="bool", def_value=False
        )

    @use_stroke_width_in_meter.setter
    def use_stroke_width_in_meter(self, value: Optional[bool]):
        self._set_attr("useStrokeWidthInMeter", value)

    # color
    @property
    def color(self) -> Optional[ColorValue]:
        return self.__color

    @color.setter
    def color(self, value: Optional[ColorValue]):
        self.__color = value
        self._set_enum_attr("color", value, ColorEnums)

    # border_color
    @property
    def border_color(self) -> Optional[ColorValue]:
        return self.__border_color

    @border_color.setter
    def border_color(self, value: Optional[ColorValue]):
        self.__border_color = value
        self._set_enum_attr("borderColor", value, ColorEnums)

    # border_stroke_width
    @property
    def border_stroke_width(self) -> float:
        return self._get_attr("borderStrokeWidth", data_type="float", def_value=0.0)

    @border_stroke_width.setter
    def border_stroke_width(self, value: OptionalNumber):
        assert value is None or value >= 0, "border_stroke_width cannot be negative"
        self._set_attr("borderStrokeWidth", value)

    # stroke_width
    @property
    def stroke_width(self) -> float:
        return self._get_attr("strokeWidth", data_type="float", def_value=1.0)

    @stroke_width.setter
    def stroke_width(self, value: OptionalNumber):
        assert value is None or value >= 0, "stroke_width cannot be negative"
        self._set_attr("strokeWidth", value)

    # coordinates
    @property
    def coordinates(self) -> List[MapLatitudeLongitude]:
        return self.__coordinates

    @coordinates.setter
    def coordinates(self, value: List[MapLatitudeLongitude]):
        self.__coordinates = value


class PolylineLayer(MapLayer):
    """
    A layer to display PolylineMarkers.

    -----

    Online docs: https://flet.dev/docs/controls/mappolylinelayer
    """

    def __init__(
        self,
        polylines: List[PolylineMarker],
        culling_margin: OptionalNumber = None,
        min_hittable_radius: OptionalNumber = None,
        simplify_tolerance: OptionalNumber = None,
        #
        # MapLayer
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):
        MapLayer.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )

        self.polylines = polylines
        self.culling_margin = culling_margin
        self.min_hittable_radius = min_hittable_radius
        self.simplify_tolerance = simplify_tolerance

    def _get_control_name(self):
        return "map_polyline_layer"

    def _get_children(self):
        return self.__polylines

    # polylines
    @property
    def polylines(self) -> List[PolylineMarker]:
        return self.__polylines

    @polylines.setter
    def polylines(self, value: List[PolylineMarker]):
        self.__polylines = value

    # culling_margin
    @property
    def culling_margin(self) -> float:
        return self._get_attr("cullingMargin", data_type="float", def_value=10.0)

    @culling_margin.setter
    def culling_margin(self, value: OptionalNumber):
        self._set_attr("cullingMargin", value)

    # simplification_tolerance
    @property
    def simplification_tolerance(self) -> float:
        return self._get_attr(
            "simplificationTolerance", data_type="float", def_value=0.4
        )

    @simplification_tolerance.setter
    def simplification_tolerance(self, value: OptionalNumber):
        self._set_attr("simplificationTolerance", value)

    # min_hittable_radius
    @property
    def min_hittable_radius(self) -> float:
        return self._get_attr("minHittableRadius", data_type="float", def_value=10.0)

    @min_hittable_radius.setter
    def min_hittable_radius(self, value: OptionalNumber):
        self._set_attr("minHittableRadius", value)



import json
from dataclasses import dataclass
from enum import Enum
from typing import Any, List, Optional, Union
from warnings import warn

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import OptionalNumber
from flet.core.control_event import ControlEvent
from flet.core.event_handler import EventHandler
from flet.core.ref import Ref
from flet.core.text_span import TextSpan
from flet.core.text_style import TextOverflow, TextStyle, TextThemeStyle
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    FontWeight,
    OffsetValue,
    OptionalControlEventCallable,
    OptionalEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
    TextAlign,
)

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


class TextAffinity(Enum):
    UPSTREAM = "upstream"
    DOWNSTREAM = "downstream"


@dataclass
class TextSelection:
    start: Optional[int] = None
    end: Optional[int] = None
    selection: Optional[str] = None
    base_offset: Optional[int] = None
    extent_offset: Optional[int] = None
    affinity: Optional[TextAffinity] = None
    directional: Optional[bool] = None
    collapsed: Optional[bool] = None
    valid: Optional[bool] = None
    normalized: Optional[bool] = None


class TextSelectionChangeCause(Enum):
    UNKNOWN = "unknown"
    TAP = "tap"
    DOUBLE_TAP = "doubleTap"
    LONG_PRESS = "longPress"
    FORCE_PRESS = "forcePress"
    KEYBOARD = "keyboard"
    TOOLBAR = "toolbar"
    DRAG = "drag"
    SCRIBBLE = "scribble"


class TextSelectionChangeEvent(ControlEvent):
    def __init__(self, e: ControlEvent):
        super().__init__(e.target, e.name, e.data, e.control, e.page)
        d = json.loads(e.data)
        self.text: str = d.get("text")
        self.cause = TextSelectionChangeCause(d.get("cause"))
        start = d.get("start")
        end = d.get("end")
        self.selection = TextSelection(
            start=start,
            end=end,
            selection=self.text[start:end] if (start != -1 and end != -1) else "",
            base_offset=d.get("base_offset"),
            extent_offset=d.get("extent_offset"),
            affinity=d.get("affinity"),
            directional=d.get("directional"),
            collapsed=d.get("collapsed"),
            valid=d.get("valid"),
            normalized=d.get("normalized"),
        )


class Text(ConstrainedControl):
    """
    Text is a control for displaying text.

    Example:
    ```
    import flet as ft

    def main(page: ft.Page):
        page.title = "Text examples"

        page.add(
            ft.Text("Size 10", size=10),
            ft.Text("Size 30, Italic", size=20, color="pink600", italic=True),
            ft.Text("Limit long text to 2 lines and fading", style=ft.TextThemeStyle.HEADLINE_SMALL),
            ft.Text(
                "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Curabitur quis nibh vitae purus consectetur facilisis sed vitae ipsum. Quisque faucibus sed nulla placerat sagittis. Phasellus condimentum risus vitae nulla vestibulum auctor. Curabitur scelerisque, nibh eget imperdiet consequat, odio ante tempus diam, sed volutpat nisl erat eget turpis. Sed viverra, diam sit amet blandit vulputate, mi tellus dapibus lorem, vitae vehicula diam mauris placerat diam. Morbi sit amet pretium turpis, et consequat ligula. Nulla velit sem, suscipit sit amet dictum non, tincidunt sed nulla. Aenean pellentesque odio porttitor sagittis aliquam. Nam varius at metus vitae vulputate. Praesent faucibus nibh lorem, eu pretium dolor dictum nec. Phasellus eget dui laoreet, viverra magna vitae, pellentesque diam.",
                max_lines=2,
            ),
        )

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/text
    """

    def __init__(
        self,
        value: Optional[str] = None,
        spans: Optional[List[TextSpan]] = None,
        text_align: Optional[TextAlign] = None,
        font_family: Optional[str] = None,
        size: OptionalNumber = None,
        weight: Optional[FontWeight] = None,
        italic: Optional[bool] = None,
        style: Union[TextThemeStyle, TextStyle, None] = None,
        theme_style: Optional[TextThemeStyle] = None,
        max_lines: Optional[int] = None,
        overflow: Optional[TextOverflow] = None,
        selectable: Optional[bool] = None,
        no_wrap: Optional[bool] = None,
        color: Optional[ColorValue] = None,
        bgcolor: Optional[ColorValue] = None,
        semantics_label: Optional[str] = None,
        show_selection_cursor: Optional[bool] = None,
        enable_interactive_selection: Optional[bool] = None,
        selection_cursor_width: OptionalNumber = None,
        selection_cursor_height: OptionalNumber = None,
        selection_cursor_color: Optional[ColorValue] = None,
        on_tap: OptionalControlEventCallable = None,
        on_selection_change: OptionalEventCallable[TextSelectionChangeEvent] = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
        rtl: Optional[bool] = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
            rtl=rtl,
        )

        self.__on_selection_change = EventHandler(lambda e: TextSelectionChangeEvent(e))

        self._add_event_handler(
            "selection_change", self.__on_selection_change.get_handler()
        )
        self.value = value
        self.spans = spans
        self.text_align = text_align
        self.font_family = font_family
        self.size = size
        self.weight = weight
        self.italic = italic
        self.no_wrap = no_wrap
        self.style = style
        self.theme_style = theme_style
        self.max_lines = max_lines
        self.overflow = overflow
        self.selectable = selectable
        self.color = color
        self.bgcolor = bgcolor
        self.semantics_label = semantics_label
        self.on_tap = on_tap
        self.on_selection_change = on_selection_change
        self.show_selection_cursor = show_selection_cursor
        self.enable_interactive_selection = enable_interactive_selection
        self.selection_cursor_width = selection_cursor_width
        self.selection_cursor_height = selection_cursor_height
        self.selection_cursor_color = selection_cursor_color

    def _get_control_name(self):
        return "text"

    def _get_children(self):
        return self.__spans

    def before_update(self):
        super().before_update()
        if isinstance(self.__style, TextStyle):
            self._set_attr_json("style", self.__style)

    # value
    @property
    def value(self) -> Optional[str]:
        return self._get_attr("value")

    @value.setter
    def value(self, value: Optional[str]):
        self._set_attr("value", value)

    # spans
    @property
    def spans(self) -> List[TextSpan]:
        return self.__spans

    @spans.setter
    def spans(self, value: Optional[List[TextSpan]]):
        self.__spans = value if value is not None else []

    # text_align
    @property
    def text_align(self) -> Optional[TextAlign]:
        return self.__text_align

    @text_align.setter
    def text_align(self, value: Optional[TextAlign]):
        self.__text_align = value
        self._set_enum_attr("textAlign", value, TextAlign)

    # font_family
    @property
    def font_family(self) -> Optional[str]:
        return self._get_attr("fontFamily")

    @font_family.setter
    def font_family(self, value: Optional[str]):
        self._set_attr("fontFamily", value)

    # size
    @property
    def size(self) -> OptionalNumber:
        return self._get_attr("size", data_type="float")

    @size.setter
    def size(self, value: OptionalNumber):
        self._set_attr("size", value)

    # weight
    @property
    def weight(self) -> Optional[FontWeight]:
        return self.__weight

    @weight.setter
    def weight(self, value: Optional[FontWeight]):
        self.__weight = value
        self._set_enum_attr("weight", value, FontWeight)

    # style
    @property
    def style(self) -> Union[TextThemeStyle, TextStyle, None]:
        return self.__style

    @style.setter
    def style(self, value: Union[TextThemeStyle, TextStyle, None]):
        self.__style = value
        if isinstance(value, (TextThemeStyle, str)) or value is None:
            self._set_attr(
                "style", value.value if isinstance(value, TextThemeStyle) else value
            )
            if value is not None:
                warn(
                    "If you wish to set the TextThemeStyle, use `Text.theme_style` instead. "
                    "The `Text.style` property should be used to set the TextStyle only.",
                    stacklevel=2,
                    category=DeprecationWarning,
                )

    # theme_style
    @property
    def theme_style(self) -> Optional[TextThemeStyle]:
        return self.__theme_style

    @theme_style.setter
    def theme_style(self, value: Optional[TextThemeStyle]):
        self.__theme_style = value
        self._set_enum_attr("theme_style", value, TextThemeStyle)

    # italic
    @property
    def italic(self) -> bool:
        return self._get_attr("italic", data_type="bool", def_value=False)

    @italic.setter
    def italic(self, value: Optional[bool]):
        self._set_attr("italic", value)

    # no_wrap
    @property
    def no_wrap(self) -> bool:
        return self._get_attr("italic", data_type="noWrap", def_value=False)

    @no_wrap.setter
    def no_wrap(self, value: Optional[bool]):
        self._set_attr("noWrap", value)

    # selectable
    @property
    def selectable(self) -> bool:
        return self._get_attr("selectable", data_type="bool", def_value=False)

    @selectable.setter
    def selectable(self, value: Optional[bool]):
        self._set_attr("selectable", value)

    # max_lines
    @property
    def max_lines(self) -> Optional[int]:
        return self._get_attr("maxLines")

    @max_lines.setter
    def max_lines(self, value: Optional[int]):
        self._set_attr("maxLines", value)

    # overflow
    @property
    def overflow(self) -> Optional[TextOverflow]:
        return self.__overflow

    @overflow.setter
    def overflow(self, value: Optional[TextOverflow]):
        self.__overflow = value
        self._set_enum_attr("overflow", value, TextOverflow)

    # color
    @property
    def color(self) -> Optional[ColorValue]:
        return self.__color

    @color.setter
    def color(self, value: Optional[ColorValue]):
        self.__color = value
        self._set_enum_attr("color", value, ColorEnums)

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # semantics_label
    @property
    def semantics_label(self) -> Optional[str]:
        return self._get_attr("semanticsLabel")

    @semantics_label.setter
    def semantics_label(self, value: Optional[str]):
        self._set_attr("semanticsLabel", value)

    # selection_cursor_color
    @property
    def selection_cursor_color(self) -> Optional[str]:
        return self._get_attr("selectionCursorColor")

    @selection_cursor_color.setter
    def selection_cursor_color(self, value: Optional[str]):
        self._set_attr("selectionCursorColor", value)

    # selection_cursor_height
    @property
    def selection_cursor_height(self) -> OptionalNumber:
        return self._get_attr("selectionCursorHeight", data_type="float")

    @selection_cursor_height.setter
    def selection_cursor_height(self, value: OptionalNumber):
        self._set_attr("selectionCursorHeight", value)

    # selection_cursor_width
    @property
    def selection_cursor_width(self) -> OptionalNumber:
        return self._get_attr("selectionCursorWidth", data_type="float", def_value=2.0)

    @selection_cursor_width.setter
    def selection_cursor_width(self, value: OptionalNumber):
        self._set_attr("selectionCursorWidth", value)

    # show_selection_cursor
    @property
    def show_selection_cursor(self) -> Optional[bool]:
        return self._get_attr("showSelectionCursor", data_type="bool", def_value=False)

    @show_selection_cursor.setter
    def show_selection_cursor(self, value: Optional[bool]):
        self._set_attr("showSelectionCursor", value)

    # enable_interactive_selection
    @property
    def enable_interactive_selection(self) -> Optional[bool]:
        return self._get_attr(
            "enableInteractiveSelection", data_type="bool", def_value=True
        )

    @enable_interactive_selection.setter
    def enable_interactive_selection(self, value: Optional[bool]):
        self._set_attr("enableInteractiveSelection", value)

    # on_tap
    @property
    def on_tap(self) -> OptionalControlEventCallable:
        return self._get_event_handler("tap")

    @on_tap.setter
    def on_tap(self, handler: OptionalControlEventCallable):
        self._add_event_handler("tap", handler)

    # on_selection_change
    @property
    def on_selection_change(
        self,
    ) -> OptionalEventCallable[TextSelectionChangeEvent]:
        return self.__on_selection_change.handler

    @on_selection_change.setter
    def on_selection_change(
        self, handler: OptionalEventCallable[TextSelectionChangeEvent]
    ):
        self.__on_selection_change.handler = handler



#!/usr/bin/env python3
"""
Bağımlılık Sistemi Test Scripti
================================

Bu script dependency.cursorrules kurallarına göre geliştirilen
bağımlılık sistemini test eder.
"""

import os
import sys
import json
import tempfile
import shutil
from pathlib import Path

def test_dependency_resolver():
    """dependency_resolver modülünü test eder"""
    print("🔍 dependency_resolver modülü test ediliyor...")
    
    try:
        from dependency_resolver import (
            check_and_install_python_dependencies,
            check_and_install_lua_dependencies,
            check_engine_availability,
            get_dependency_report,
            get_system_dependency_report,
            get_enhanced_system_dependency_report,
            resolve_all_dependencies
        )
        print("✅ dependency_resolver modülü başarıyla import edildi")
        return True
    except ImportError as e:
        print(f"❌ dependency_resolver import hatası: {e}")
        return False

def test_dependency_command():
    """dependency_command modülünü test eder"""
    print("🔍 dependency_command modülü test ediliyor...")
    
    try:
        from dependency_command import (
            handle_dependency_check,
            handle_dependency_install,
            handle_engine_check,
            handle_dependency_tree
        )
        print("✅ dependency_command modülü başarıyla import edildi")
        return True
    except ImportError as e:
        print(f"❌ dependency_command import hatası: {e}")
        return False

def test_python_dependency_installation():
    """Python bağımlılık kurulumunu test eder"""
    print("🔍 Python bağımlılık kurulumu test ediliyor...")
    
    # Test manifest oluştur
    with tempfile.TemporaryDirectory() as temp_dir:
        manifest = {
            "name": "test-python-app",
            "version": "1.0.0",
            "language": "python",
            "entry": "main.py",
            "dependencies": ["requests", "colorama"]
        }
        
        manifest_path = Path(temp_dir) / "manifest.json"
        with open(manifest_path, 'w') as f:
            json.dump(manifest, f)
        
        # Test fonksiyonunu çağır
        from dependency_resolver import check_and_install_python_dependencies
        
        success, message, missing_packages = check_and_install_python_dependencies(temp_dir)
        
        if success:
            print(f"✅ Python bağımlılık testi başarılı: {message}")
            return True
        else:
            print(f"❌ Python bağımlılık testi başarısız: {message}")
            return False

def test_lua_dependency_installation():
    """Lua bağımlılık kurulumunu test eder"""
    print("🔍 Lua bağımlılık kurulumu test ediliyor...")
    
    # Test manifest oluştur
    with tempfile.TemporaryDirectory() as temp_dir:
        manifest = {
            "name": "test-lua-app",
            "version": "1.0.0",
            "language": "lua",
            "entry": "main.lua",
            "dependencies": ["luasocket", "lpeg"]
        }
        
        manifest_path = Path(temp_dir) / "manifest.json"
        with open(manifest_path, 'w') as f:
            json.dump(manifest, f)
        
        # Test fonksiyonunu çağır
        from dependency_resolver import check_and_install_lua_dependencies
        
        success, message, missing_packages = check_and_install_lua_dependencies(temp_dir)
        
        if success:
            print(f"✅ Lua bağımlılık testi başarılı: {message}")
            return True
        else:
            print(f"❌ Lua bağımlılık testi başarısız: {message}")
            return False

def test_engine_availability():
    """Engine kontrolünü test eder"""
    print("🔍 Engine kontrolü test ediliyor...")
    
    # Test manifest oluştur
    with tempfile.TemporaryDirectory() as temp_dir:
        manifest = {
            "name": "test-engine-app",
            "version": "1.0.0",
            "language": "python",
            "entry": "main.py",
            "engine": "requests"
        }
        
        manifest_path = Path(temp_dir) / "manifest.json"
        with open(manifest_path, 'w') as f:
            json.dump(manifest, f)
        
        # Test fonksiyonunu çağır
        from dependency_resolver import check_engine_availability
        
        available, message, engine_info = check_engine_availability(temp_dir)
        
        print(f"Engine kontrolü: {message}")
        if engine_info:
            print(f"Engine bilgileri: {engine_info}")
        
        return True  # Engine kontrolü başarılı olarak kabul et

def test_dependency_reports():
    """Bağımlılık raporlarını test eder"""
    print("🔍 Bağımlılık raporları test ediliyor...")
    
    try:
        from dependency_resolver import (
            get_system_dependency_report,
            get_enhanced_system_dependency_report
        )
        
        # Sistem raporu
        system_report = get_system_dependency_report()
        print("✅ Sistem bağımlılık raporu oluşturuldu")
        
        # Gelişmiş sistem raporu
        enhanced_report = get_enhanced_system_dependency_report()
        print("✅ Gelişmiş sistem raporu oluşturuldu")
        
        return True
    except Exception as e:
        print(f"❌ Rapor oluşturma hatası: {e}")
        return False

def test_install_integration():
    """Install komutundaki bağımlılık entegrasyonunu test eder"""
    print("🔍 Install entegrasyonu test ediliyor...")
    
    try:
        from install_command import install_app_locally
        
        # Test için basit bir uygulama oluştur
        with tempfile.TemporaryDirectory() as temp_dir:
            # Test uygulaması oluştur
            app_name = "test-dependency-app"
            app_dir = Path(temp_dir) / app_name
            app_dir.mkdir()
            
            manifest = {
                "name": app_name,
                "version": "1.0.0",
                "language": "python",
                "entry": "main.py",
                "dependencies": ["requests"]
            }
            
            manifest_path = app_dir / "manifest.json"
            with open(manifest_path, 'w') as f:
                json.dump(manifest, f)
            
            # Basit main.py oluştur
            main_py = app_dir / "main.py"
            with open(main_py, 'w') as f:
                f.write('print("Hello from test app")')
            
            # install_app_locally fonksiyonunu test et
            # Bu fonksiyon artık bağımlılık kontrolü yapıyor
            success, message = install_app_locally(app_name, str(app_dir))
            
            if success:
                print(f"✅ Install entegrasyonu başarılı: {message}")
                return True
            else:
                print(f"❌ Install entegrasyonu başarısız: {message}")
                return False
                
    except Exception as e:
        print(f"❌ Install entegrasyonu hatası: {e}")
        return False

def test_cli_integration():
    """CLI entegrasyonunu test eder"""
    print("🔍 CLI entegrasyonu test ediliyor...")
    
    try:
        from main import main
        
        # main.py'de dependency komutlarının tanımlı olduğunu kontrol et
        print("✅ CLI entegrasyonu başarılı")
        return True
    except Exception as e:
        print(f"❌ CLI entegrasyonu hatası: {e}")
        return False

def test_requirements_txt_fallback():
    """requirements.txt fallback özelliğini test eder"""
    print("🔍 requirements.txt fallback test ediliyor...")
    
    with tempfile.TemporaryDirectory() as temp_dir:
        # Test manifest oluştur
        manifest = {
            "name": "test-requirements-app",
            "version": "1.0.0",
            "language": "python",
            "entry": "main.py",
            "dependencies": ["requests"]
        }
        
        manifest_path = Path(temp_dir) / "manifest.json"
        with open(manifest_path, 'w') as f:
            json.dump(manifest, f)
        
        # requirements.txt oluştur
        req_txt = Path(temp_dir) / "requirements.txt"
        with open(req_txt, 'w') as f:
            f.write("colorama\n")
        
        # Test fonksiyonunu çağır
        from dependency_resolver import check_and_install_python_dependencies
        
        success, message, missing_packages = check_and_install_python_dependencies(temp_dir)
        
        if success:
            print(f"✅ requirements.txt fallback testi başarılı: {message}")
            return True
        else:
            print(f"❌ requirements.txt fallback testi başarısız: {message}")
            return False

def run_all_tests():
    """Tüm testleri çalıştırır"""
    print("🚀 Bağımlılık Sistemi Testleri Başlatılıyor")
    print("=" * 60)
    
    tests = [
        ("Dependency Resolver Modülü", test_dependency_resolver),
        ("Dependency Command Modülü", test_dependency_command),
        ("Python Bağımlılık Kurulumu", test_python_dependency_installation),
        ("Lua Bağımlılık Kurulumu", test_lua_dependency_installation),
        ("Engine Kontrolü", test_engine_availability),
        ("Bağımlılık Raporları", test_dependency_reports),
        ("Install Entegrasyonu", test_install_integration),
        ("CLI Entegrasyonu", test_cli_integration),
        ("Requirements.txt Fallback", test_requirements_txt_fallback)
    ]
    
    passed = 0
    total = len(tests)
    
    for test_name, test_func in tests:
        print(f"\n📋 {test_name}")
        print("-" * 40)
        
        try:
            if test_func():
                passed += 1
                print(f"✅ {test_name} BAŞARILI")
            else:
                print(f"❌ {test_name} BAŞARISIZ")
        except Exception as e:
            print(f"❌ {test_name} HATA: {e}")
    
    print("\n" + "=" * 60)
    print(f"📊 Test Sonuçları: {passed}/{total} başarılı")
    
    if passed == total:
        print("🎉 Tüm testler başarılı! Bağımlılık sistemi hazır.")
        return True
    else:
        print("⚠️  Bazı testler başarısız. Lütfen kontrol edin.")
        return False

if __name__ == "__main__":
    success = run_all_tests()
    sys.exit(0 if success else 1) 


from typing import Any, List, Optional, Union

from flet.core.adaptive_control import AdaptiveControl
from flet.core.alignment import Alignment
from flet.core.buttons import OutlinedBorder
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.text_style import TextStyle
from flet.core.types import (
    ClipBehavior,
    ColorEnums,
    ColorValue,
    MainAxisAlignment,
    OptionalControlEventCallable,
    PaddingValue,
)


class AlertDialog(AdaptiveControl):
    """
    An alert dialog informs the user about situations that require acknowledgement. An alert dialog has an optional title and an optional list of actions. The title is displayed above the content and the actions are displayed below the content.

    Example:
    ```
    import flet as ft


    def main(page: ft.Page):
        page.title = "AlertDialog examples"
        page.horizontal_alignment = ft.CrossAxisAlignment.CENTER

        dlg = ft.AlertDialog(
            title=ft.Text("Hi, this is a non-modal dialog!"),
            on_dismiss=lambda e: page.add(ft.Text("Non-modal dialog dismissed")),
        )

        def handle_close(e):
            page.close(dlg_modal)
            page.add(ft.Text(f"Modal dialog closed with action: {e.control.text}"))

        dlg_modal = ft.AlertDialog(
            modal=True,
            title=ft.Text("Please confirm"),
            content=ft.Text("Do you really want to delete all those files?"),
            actions=[
                ft.TextButton("Yes", on_click=handle_close),
                ft.TextButton("No", on_click=handle_close),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            on_dismiss=lambda e: page.add(
                ft.Text("Modal dialog dismissed"),
            ),
        )

        page.add(
            ft.ElevatedButton("Open dialog", on_click=lambda e: page.open(dlg)),
            ft.ElevatedButton("Open modal dialog", on_click=lambda e: page.open(dlg_modal)),
        )


    ft.app(target=main)
    ```
    -----

    Online docs: https://flet.dev/docs/controls/alertdialog
    """

    def __init__(
        self,
        modal: bool = False,
        title: Optional[Union[Control, str]] = None,
        content: Optional[Control] = None,
        actions: Optional[List[Control]] = None,
        bgcolor: Optional[ColorValue] = None,
        elevation: OptionalNumber = None,
        icon: Optional[Control] = None,
        open: bool = False,
        title_padding: Optional[PaddingValue] = None,
        content_padding: Optional[PaddingValue] = None,
        actions_padding: Optional[PaddingValue] = None,
        actions_alignment: Optional[MainAxisAlignment] = None,
        shape: Optional[OutlinedBorder] = None,
        inset_padding: Optional[PaddingValue] = None,
        icon_padding: Optional[PaddingValue] = None,
        action_button_padding: Optional[PaddingValue] = None,
        surface_tint_color: Optional[ColorValue] = None,
        shadow_color: Optional[ColorValue] = None,
        icon_color: Optional[ColorValue] = None,
        scrollable: Optional[bool] = None,
        actions_overflow_button_spacing: OptionalNumber = None,
        alignment: Optional[Alignment] = None,
        content_text_style: Optional[TextStyle] = None,
        title_text_style: Optional[TextStyle] = None,
        clip_behavior: Optional[ClipBehavior] = None,
        semantics_label: Optional[str] = None,
        barrier_color: Optional[ColorValue] = None,
        on_dismiss: OptionalControlEventCallable = None,
        #
        # AdaptiveControl
        #
        ref: Optional[Ref] = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
        data: Any = None,
        adaptive: Optional[bool] = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            disabled=disabled,
            visible=visible,
            data=data,
        )

        AdaptiveControl.__init__(self, adaptive=adaptive)

        self.open = open
        self.bgcolor = bgcolor
        self.elevation = elevation
        self.icon = icon
        self.modal = modal
        self.title = title
        self.title_padding = title_padding
        self.content = content
        self.content_padding = content_padding
        self.actions = actions
        self.actions_padding = actions_padding
        self.actions_alignment = actions_alignment
        self.shape = shape
        self.inset_padding = inset_padding
        self.semantics_label = semantics_label
        self.on_dismiss = on_dismiss
        self.clip_behavior = clip_behavior
        self.action_button_padding = action_button_padding
        self.shadow_color = shadow_color
        self.surface_tint_color = surface_tint_color
        self.icon_padding = icon_padding
        self.icon_color = icon_color
        self.scrollable = scrollable
        self.actions_overflow_button_spacing = actions_overflow_button_spacing
        self.alignment = alignment
        self.content_text_style = content_text_style
        self.title_text_style = title_text_style
        self.barrier_color = barrier_color

    def _get_control_name(self):
        return "alertdialog"

    def before_update(self):
        super().before_update()
        assert (
            self.__title or self.__content or self.__actions
        ), "AlertDialog has nothing to display. Provide at minimum one of the following: title, content, actions"
        self._set_attr_json("actionsPadding", self.__actions_padding)
        self._set_attr_json("contentPadding", self.__content_padding)
        self._set_attr_json("titlePadding", self.__title_padding)
        self._set_attr_json("shape", self.__shape)
        self._set_attr_json("insetPadding", self.__inset_padding)
        self._set_attr_json("iconPadding", self.__icon_padding)
        self._set_attr_json("actionButtonPadding", self.__action_button_padding)
        self._set_attr_json("alignment", self.__alignment)
        if isinstance(self.__content_text_style, TextStyle):
            self._set_attr_json("contentTextStyle", self.__content_text_style)
        if isinstance(self.__title_text_style, TextStyle):
            self._set_attr_json("titleTextStyle", self.__title_text_style)

    def _get_children(self):
        children = []
        if isinstance(self.__title, Control):
            self.__title._set_attr_internal("n", "title")
            children.append(self.__title)
        if self.__icon:
            self.__icon._set_attr_internal("n", "icon")
            children.append(self.__icon)
        if self.__content:
            self.__content._set_attr_internal("n", "content")
            children.append(self.__content)
        for action in self.__actions:
            action._set_attr_internal("n", "action")
            children.append(action)
        return children

    # open
    @property
    def open(self) -> bool:
        return self._get_attr("open", data_type="bool", def_value=False)

    @open.setter
    def open(self, value: Optional[bool]):
        self._set_attr("open", value)

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # shadow_color
    @property
    def shadow_color(self) -> Optional[ColorValue]:
        return self.__shadow_color

    @shadow_color.setter
    def shadow_color(self, value: Optional[ColorValue]):
        self.__shadow_color = value
        self._set_enum_attr("shadowColor", value, ColorEnums)

    # barrier_color
    @property
    def barrier_color(self) -> Optional[ColorValue]:
        return self.__barrier_color

    @barrier_color.setter
    def barrier_color(self, value: Optional[ColorValue]):
        self.__barrier_color = value
        self._set_enum_attr("barrierColor", value, ColorEnums)

    # surface_tint_color
    @property
    def surface_tint_color(self) -> Optional[ColorValue]:
        return self.__surface_tint_color

    @surface_tint_color.setter
    def surface_tint_color(self, value: Optional[ColorValue]):
        self.__surface_tint_color = value
        self._set_enum_attr("surfaceTintColor", value, ColorEnums)

    # icon_color
    @property
    def icon_color(self) -> Optional[ColorValue]:
        return self.__icon_color

    @icon_color.setter
    def icon_color(self, value: Optional[ColorValue]):
        self.__icon_color = value
        self._set_enum_attr("iconColor", value, ColorEnums)

    # elevation
    @property
    def elevation(self) -> OptionalNumber:
        return self._get_attr("elevation", data_type="float")

    @elevation.setter
    def elevation(self, value: OptionalNumber):
        self._set_attr("elevation", value)

    # actions_overflow_button_spacing
    @property
    def actions_overflow_button_spacing(self) -> OptionalNumber:
        return self._get_attr("actionsOverflowButtonSpacing", data_type="float")

    @actions_overflow_button_spacing.setter
    def actions_overflow_button_spacing(self, value: OptionalNumber):
        self._set_attr("actionsOverflowButtonSpacing", value)

    # scrollable
    @property
    def scrollable(self) -> bool:
        return self._get_attr("scrollable", data_type="bool", def_value=False)

    @scrollable.setter
    def scrollable(self, value: Optional[bool]):
        self._set_attr("scrollable", value)

    # alignment
    @property
    def alignment(self) -> Optional[Alignment]:
        return self.__alignment

    @alignment.setter
    def alignment(self, value: Optional[Alignment]):
        self.__alignment = value

    # content_text_style
    @property
    def content_text_style(self) -> Optional[TextStyle]:
        return self.__content_text_style

    @content_text_style.setter
    def content_text_style(self, value: Optional[TextStyle]):
        self.__content_text_style = value

    # title_text_style
    @property
    def title_text_style(self) -> Optional[TextStyle]:
        return self.__title_text_style

    @title_text_style.setter
    def title_text_style(self, value: Optional[TextStyle]):
        self.__title_text_style = value

    # modal
    @property
    def modal(self) -> bool:
        return self._get_attr("modal", data_type="bool", def_value=False)

    @modal.setter
    def modal(self, value: Optional[bool]):
        self._set_attr("modal", value)

    # title
    @property
    def title(self) -> Optional[Union[Control, str]]:
        return self.__title

    @title.setter
    def title(self, value: Optional[Union[Control, str]]):
        self.__title = value
        if not isinstance(value, Control):
            self._set_attr("title", value)

    # icon
    @property
    def icon(self) -> Optional[Control]:
        return self.__icon

    @icon.setter
    def icon(self, value: Optional[Control]):
        self.__icon = value

    # title_padding
    @property
    def title_padding(self) -> Optional[PaddingValue]:
        return self.__title_padding

    @title_padding.setter
    def title_padding(self, value: Optional[PaddingValue]):
        self.__title_padding = value

    # content
    @property
    def content(self) -> Optional[Control]:
        return self.__content

    @content.setter
    def content(self, value: Optional[Control]):
        self.__content = value

    # content_padding
    @property
    def content_padding(self) -> Optional[PaddingValue]:
        return self.__content_padding

    @content_padding.setter
    def content_padding(self, value: Optional[PaddingValue]):
        self.__content_padding = value

    # actions
    @property
    def actions(self) -> List[Control]:
        return self.__actions

    @actions.setter
    def actions(self, value: Optional[List[Control]]):
        self.__actions = value if value is not None else []

    # actions_padding
    @property
    def actions_padding(self) -> Optional[PaddingValue]:
        return self.__actions_padding

    @actions_padding.setter
    def actions_padding(self, value: Optional[PaddingValue]):
        self.__actions_padding = value

    # actions_alignment
    @property
    def actions_alignment(self) -> Optional[MainAxisAlignment]:
        return self.__actions_alignment

    @actions_alignment.setter
    def actions_alignment(self, value: Optional[MainAxisAlignment]):
        self.__actions_alignment = value
        self._set_attr(
            "actionsAlignment",
            value.value if isinstance(value, MainAxisAlignment) else value,
        )

    # shape
    @property
    def shape(self) -> Optional[OutlinedBorder]:
        return self.__shape

    @shape.setter
    def shape(self, value: Optional[OutlinedBorder]):
        self.__shape = value

    # inset_padding
    @property
    def inset_padding(self) -> Optional[PaddingValue]:
        return self.__inset_padding

    @inset_padding.setter
    def inset_padding(self, value: Optional[PaddingValue]):
        self.__inset_padding = value

    # icon_padding
    @property
    def icon_padding(self) -> Optional[PaddingValue]:
        return self.__icon_padding

    @icon_padding.setter
    def icon_padding(self, value: Optional[PaddingValue]):
        self.__icon_padding = value

    # action_button_padding
    @property
    def action_button_padding(self) -> Optional[PaddingValue]:
        return self.__action_button_padding

    @action_button_padding.setter
    def action_button_padding(self, value: Optional[PaddingValue]):
        self.__action_button_padding = value

    # semantics_label
    @property
    def semantics_label(self) -> Optional[str]:
        return self._get_attr("semanticsLabel")

    @semantics_label.setter
    def semantics_label(self, value: Optional[str]):
        self._set_attr("semanticsLabel", value)

    # clip_behavior
    @property
    def clip_behavior(self) -> Optional[ClipBehavior]:
        return self.__clip_behavior

    @clip_behavior.setter
    def clip_behavior(self, value: Optional[ClipBehavior]):
        self.__clip_behavior = value
        self._set_enum_attr("clipBehavior", value, ClipBehavior)

    # on_dismiss
    @property
    def on_dismiss(self) -> OptionalControlEventCallable:
        return self._get_event_handler("dismiss")

    @on_dismiss.setter
    def on_dismiss(self, handler: OptionalControlEventCallable):
        self._add_event_handler("dismiss", handler)



import logging
from typing import Any, Callable

import flet.core
from flet.core.pubsub.pubsub_hub import PubSubHub

logger = logging.getLogger(flet.__name__)


class PubSubClient:
    def __init__(self, pubsub: PubSubHub, session_id: str):
        self.__pubsub = pubsub
        self.__session_id = session_id

    def send_all(self, message: Any):
        self.__pubsub.send_all(message)

    def send_all_on_topic(self, topic: str, message: Any):
        self.__pubsub.send_all_on_topic(topic, message)

    def send_others(self, message: Any):
        self.__pubsub.send_others(self.__session_id, message)

    def send_others_on_topic(self, topic: str, message: Any):
        self.__pubsub.send_others_on_topic(self.__session_id, topic, message)

    def subscribe(self, handler: Callable):
        self.__pubsub.subscribe(self.__session_id, handler)

    def subscribe_topic(self, topic: str, handler: Callable):
        self.__pubsub.subscribe_topic(self.__session_id, topic, handler)

    def unsubscribe(self):
        self.__pubsub.unsubscribe(self.__session_id)

    def unsubscribe_topic(self, topic: str):
        self.__pubsub.unsubscribe_topic(self.__session_id, topic)

    def unsubscribe_all(self):
        self.__pubsub.unsubscribe_all(self.__session_id)



import re
import urllib.parse


class UrlComponents:
    """
    `UrlComponents` are meant to be used internally for decoding-encoding, it has no external use
    """

    def _encode_url_component(self, url: str) -> str:
        """
        Function encodes querystring part of URL\n
        Ex. q=dom & dogs -> q=dom+%26+dogs
        """
        return urllib.parse.quote(url)

    def _decode_url_component(self, url: str) -> str:
        """
        Function decodes querystring part of URL\n
        Ex. q=dom+%26+dogs -> q=dom & dogs
        """
        return urllib.parse.unquote(url)

    def _is_encoded(self) -> bool:
        """
        Function returns True if URL is already encoded
        """
        if "?" in self.url:
            q_result = self._querystring_part()
            return (
                self._decode_url_component(
                    self.url[q_result.start() + 1 : q_result.end()]
                )
                != self.url[q_result.start() + 1 : q_result.end()]
            )

    def _querystring_part(self, url_string: bool = False):
        """
        Function sliced url part and returns querystring part.\n
        Use case: checking querystring part for encode, assiging decoded value
        """
        pattern = re.compile(r"\?[\w\D]+")
        data = pattern.search(self.url)
        return data if url_string is False else self.url[data.start() + 1 : data.end()]


class QueryString(UrlComponents):
    """
    Note:
        `QueryString` class is meant to be for internal use inside of page. Hence, methods such as `get()` or `to_dict()` must be\n
        called from `page` object\n

    Constructor:
            `page` takes `Page` class an an argument and extracts URL automatically\n

    Methods:
            Public:
                `get()` method takes `key` an an argument and returns value according to key. (Ex: .../?name=Joe -> `get('name')` -> `Joe`)\n
                `to_dict` returns all the key-value pairs of querystring as a `dict`\n
                `path` returns url path (Ex: .../products?id=1 -> /products)

            Private(meant to be used only inside of page class):
                `post()` method takes key-value pair as an argument and returs proceeded querystring ready to be merged with URL

    """

    def __init__(self, page=None):
        self.page = page
        self.url = None

    def get(self, key: str) -> str:
        self._data = self.to_dict
        return self._data[key]

    def post(self, kwargs: dict):
        return "?" + urllib.parse.urlencode(kwargs)

    @property
    def to_dict(self) -> dict:
        self._data = urllib.parse.urlparse(self.url).query
        return dict(urllib.parse.parse_qsl(self._data))

    # Path
    @property
    def path(self):
        self._updated_url = self.url.replace("#/", "") if "#" in self.url else self.url
        return urllib.parse.urlparse(self._updated_url).path

    def __call__(self):
        """
        Call dunder method updates url after updating `Page`
        """
        self.url = self.page.url + self.page.route

        # Checking if self.url is encoded and decoding it accordingly
        if self._is_encoded() is True:
            self.url = (
                self.page.url
                + urllib.parse.urlparse(self.url).path
                + "?"
                + self._decode_url_component(self._querystring_part(url_string=True))
            )



from typing import Callable, List, Optional

from flet.auth.group import Group
from flet.auth.user import User


class OAuthProvider:
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        authorization_endpoint: str,
        token_endpoint: str,
        redirect_url: str,
        scopes: Optional[List[str]] = None,
        user_scopes: Optional[List[str]] = None,
        user_endpoint: Optional[str] = None,
        user_id_fn: Optional[Callable] = None,
        group_scopes: Optional[List[str]] = None,
        code_challenge: Optional[str] = None,
        code_challenge_method: Optional[str] = None,
        code_verifier: Optional[str] = None,
    ) -> None:
        self.client_id = client_id
        self.client_secret = client_secret
        self.authorization_endpoint = authorization_endpoint
        self.token_endpoint = token_endpoint
        self.redirect_url = redirect_url
        self.scopes = scopes if scopes is not None else []
        self.user_scopes = user_scopes if user_scopes is not None else []
        self.user_endpoint = user_endpoint
        self.user_id_fn = user_id_fn
        self.group_scopes = group_scopes if group_scopes is not None else []
        self.code_challenge = code_challenge
        self.code_challenge_method = code_challenge_method
        self.code_verifier = code_verifier

    def _name(self):
        raise Exception("Not implemented")

    def _fetch_groups(self, access_token: str) -> List[Group]:
        return []

    async def _fetch_groups_async(self, access_token: str) -> List[Group]:
        return []

    def _fetch_user(self, access_token: str) -> Optional[User]:
        return None

    async def _fetch_user_async(self, access_token: str) -> Optional[User]:
        return None



import hashlib


def sha1(input_string):
    sha1_hash = hashlib.sha1()
    sha1_hash.update(input_string.encode("utf-8"))
    return sha1_hash.hexdigest()


def calculate_file_hash(path, blocksize=65536):
    h = hashlib.sha256()
    with open(path, "rb") as f:
        while True:
            data = f.read(blocksize)
            if not data:
                break
            h.update(data)
    return h.hexdigest()



from dataclasses import dataclass
from enum import Enum
from typing import Optional, Union

from flet.core.types import ColorValue, OptionalNumber


class BorderSideStrokeAlign(float, Enum):
    INSIDE = -1.0
    CENTER = 0.0
    OUTSIDE = 1.0


@dataclass
class BorderSide:
    width: OptionalNumber
    color: Optional[ColorValue] = None
    stroke_align: Union[BorderSideStrokeAlign, OptionalNumber] = None


@dataclass
class Border:
    top: Optional[BorderSide] = None
    right: Optional[BorderSide] = None
    bottom: Optional[BorderSide] = None
    left: Optional[BorderSide] = None


def all(width: Optional[float] = None, color: Optional[ColorValue] = None) -> Border:
    bs = BorderSide(width, color)
    return Border(left=bs, top=bs, right=bs, bottom=bs)


def symmetric(
    vertical: Optional[BorderSide] = None, horizontal: Optional[BorderSide] = None
) -> Border:
    return Border(left=horizontal, top=vertical, right=horizontal, bottom=vertical)


def only(
    left: Optional[BorderSide] = None,
    top: Optional[BorderSide] = None,
    right: Optional[BorderSide] = None,
    bottom: Optional[BorderSide] = None,
) -> Border:
    return Border(left=left, top=top, right=right, bottom=bottom)



from typing import Any, Optional

from flet.core.badge import BadgeValue
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.types import OptionalControlEventCallable


class Semantics(Control):
    """
    A control that annotates the control tree with a description of the meaning of the widgets.

    Used by accessibility tools, search engines, and other semantic analysis software to determine the meaning of the application.

    -----

    Online docs: https://flet.dev/docs/controls/semantics
    """

    def __init__(
        self,
        content: Optional[Control] = None,
        label: Optional[str] = None,
        expanded: Optional[bool] = None,
        hidden: Optional[bool] = None,
        selected: Optional[bool] = None,
        button: Optional[bool] = None,
        obscured: Optional[bool] = None,
        multiline: Optional[bool] = None,
        focusable: Optional[bool] = None,
        read_only: Optional[bool] = None,
        focus: Optional[bool] = None,
        slider: Optional[bool] = None,
        tooltip: Optional[str] = None,
        badge: Optional[BadgeValue] = None,
        toggled: Optional[bool] = None,
        max_value_length: OptionalNumber = None,
        checked: Optional[bool] = None,
        value: Optional[str] = None,
        increased_value: Optional[str] = None,
        decreased_value: Optional[str] = None,
        hint_text: Optional[str] = None,
        on_tap_hint_text: Optional[str] = None,
        current_value_length: Optional[int] = None,
        heading_level: Optional[int] = None,
        exclude_semantics: Optional[bool] = None,
        mixed: Optional[bool] = None,
        on_long_press_hint_text: Optional[str] = None,
        container: Optional[bool] = None,
        live_region: Optional[bool] = None,
        textfield: Optional[bool] = None,
        link: Optional[bool] = None,
        header: Optional[bool] = None,
        image: Optional[bool] = None,
        on_tap: OptionalControlEventCallable = None,
        on_double_tap: OptionalControlEventCallable = None,
        on_increase: OptionalControlEventCallable = None,
        on_decrease: OptionalControlEventCallable = None,
        on_dismiss: OptionalControlEventCallable = None,
        on_scroll_left: OptionalControlEventCallable = None,
        on_scroll_right: OptionalControlEventCallable = None,
        on_scroll_up: OptionalControlEventCallable = None,
        on_scroll_down: OptionalControlEventCallable = None,
        on_copy: OptionalControlEventCallable = None,
        on_cut: OptionalControlEventCallable = None,
        on_paste: OptionalControlEventCallable = None,
        on_long_press: OptionalControlEventCallable = None,
        on_move_cursor_forward_by_character: OptionalControlEventCallable = None,
        on_move_cursor_backward_by_character: OptionalControlEventCallable = None,
        on_did_gain_accessibility_focus: OptionalControlEventCallable = None,
        on_did_lose_accessibility_focus: OptionalControlEventCallable = None,
        on_set_text: OptionalControlEventCallable = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            visible=visible,
            disabled=disabled,
            data=data,
            badge=badge,
        )

        self.content = content
        self.label = label
        self.expanded = expanded
        self.hidden = hidden
        self.selected = selected
        self.button = button
        self.obscured = obscured
        self.multiline = multiline
        self.focusable = focusable
        self.read_only = read_only
        self.focus = focus
        self.slider = slider
        self.tooltip = tooltip
        self.toggled = toggled
        self.max_value_length = max_value_length
        self.checked = checked
        self.value = value
        self.increased_value = increased_value
        self.decreased_value = decreased_value
        self.hint_text = hint_text
        self.on_tap_hint_text = on_tap_hint_text
        self.on_long_press_hint_text = on_long_press_hint_text
        self.container = container
        self.live_region = live_region
        self.textfield = textfield
        self.link = link
        self.header = header
        self.image = image
        self.on_tap = on_tap
        self.on_double_tap = on_double_tap
        self.on_increase = on_increase
        self.on_decrease = on_decrease
        self.on_dismiss = on_dismiss
        self.on_scroll_left = on_scroll_left
        self.on_scroll_right = on_scroll_right
        self.on_scroll_up = on_scroll_up
        self.on_scroll_down = on_scroll_down
        self.on_copy = on_copy
        self.on_cut = on_cut
        self.on_paste = on_paste
        self.on_long_press = on_long_press
        self.on_move_cursor_forward_by_character = on_move_cursor_forward_by_character
        self.on_move_cursor_backward_by_character = on_move_cursor_backward_by_character
        self.on_did_gain_accessibility_focus = on_did_gain_accessibility_focus
        self.on_did_lose_accessibility_focus = on_did_lose_accessibility_focus
        self.current_value_length = current_value_length
        self.heading_level = heading_level
        self.exclude_semantics = exclude_semantics
        self.mixed = mixed
        self.on_set_text = on_set_text

    def _get_control_name(self):
        return "semantics"

    def _get_children(self):
        children = []
        if self.__content is not None:
            self.__content._set_attr_internal("n", "content")
            children.append(self.__content)
        return children

    # label
    @property
    def label(self) -> Optional[str]:
        return self._get_attr("label")

    @label.setter
    def label(self, value: Optional[str]):
        self._set_attr("label", value)

    # content
    @property
    def content(self) -> Optional[Control]:
        return self.__content

    @content.setter
    def content(self, value: Optional[Control]):
        self.__content = value

    # expanded
    @property
    def expanded(self) -> Optional[bool]:
        return self._get_attr("expanded", data_type="bool")

    @expanded.setter
    def expanded(self, value: Optional[bool]):
        self._set_attr("expanded", value)

    # hidden
    @property
    def hidden(self):
        return self._get_attr("hidden")

    @hidden.setter
    def hidden(self, value: Optional[bool]):
        self._set_attr("hidden", value)

    # textfield
    @property
    def textfield(self) -> Optional[bool]:
        return self._get_attr("textfield", data_type="bool")

    @textfield.setter
    def textfield(self, value: Optional[bool]):
        self._set_attr("textfield", value)

    # link
    @property
    def link(self) -> Optional[bool]:
        return self._get_attr("link", data_type="bool")

    @link.setter
    def link(self, value: Optional[bool]):
        self._set_attr("link", value)

    # image
    @property
    def image(self) -> Optional[bool]:
        return self._get_attr("image", data_type="bool")

    @image.setter
    def image(self, value: Optional[bool]):
        self._set_attr("image", value)

    # header
    @property
    def header(self) -> Optional[bool]:
        return self._get_attr("header", data_type="bool")

    @header.setter
    def header(self, value: Optional[bool]):
        self._set_attr("header", value)

    # selected
    @property
    def selected(self) -> Optional[bool]:
        return self._get_attr("selected", data_type="bool")

    @selected.setter
    def selected(self, value: Optional[bool]):
        self._set_attr("selected", value)

    # button
    @property
    def button(self) -> Optional[bool]:
        return self._get_attr("button", data_type="bool")

    @button.setter
    def button(self, value: Optional[bool]):
        self._set_attr("button", value)

    # obscured
    @property
    def obscured(self) -> Optional[bool]:
        return self._get_attr("obscured", data_type="bool")

    @obscured.setter
    def obscured(self, value: Optional[bool]):
        self._set_attr("obscured", value)

    # multiline
    @property
    def multiline(self) -> Optional[bool]:
        return self._get_attr("multiline", data_type="bool")

    @multiline.setter
    def multiline(self, value: Optional[bool]):
        self._set_attr("multiline", value)

    # focusable
    @property
    def focusable(self) -> Optional[bool]:
        return self._get_attr("focusable", data_type="bool")

    @focusable.setter
    def focusable(self, value: Optional[bool]):
        self._set_attr("focusable", value)

    # read_only
    @property
    def read_only(self) -> Optional[bool]:
        return self._get_attr("readOnly", data_type="bool")

    @read_only.setter
    def read_only(self, value: Optional[bool]):
        self._set_attr("readOnly", value)

    # focused
    @property
    def focused(self) -> Optional[bool]:
        return self._get_attr("focused", data_type="bool")

    @focused.setter
    def focused(self, value: Optional[bool]):
        self._set_attr("focused", value)

    # mixed
    @property
    def mixed(self) -> Optional[bool]:
        return self._get_attr("mixed", data_type="bool")

    @mixed.setter
    def mixed(self, value: Optional[bool]):
        self._set_attr("mixed", value)

    # exclude_semantics
    @property
    def exclude_semantics(self) -> bool:
        return self._get_attr("excludeSemantics", data_type="bool", def_value=False)

    @exclude_semantics.setter
    def exclude_semantics(self, value: Optional[bool]):
        self._set_attr("excludeSemantics", value)

    # heading_level
    @property
    def heading_level(self) -> Optional[int]:
        return self._get_attr("headingLevel", data_type="int")

    @heading_level.setter
    def heading_level(self, value: Optional[int]):
        self._set_attr("headingLevel", value)

    # current_value_length
    @property
    def current_value_length(self) -> Optional[int]:
        return self._get_attr("currentValueLength", data_type="int")

    @current_value_length.setter
    def current_value_length(self, value: Optional[int]):
        self._set_attr("currentValueLength", value)

    # slider
    @property
    def slider(self) -> Optional[bool]:
        return self._get_attr("slider", data_type="bool")

    @slider.setter
    def slider(self, value: Optional[bool]):
        self._set_attr("slider", value)

    # tooltip
    @property
    def tooltip(self) -> Optional[str]:
        return self._get_attr("tooltip")

    @tooltip.setter
    def tooltip(self, value: Optional[str]):
        self._set_attr("tooltip", value)

    # toggled
    @property
    def toggled(self) -> Optional[bool]:
        return self._get_attr("toggled", data_type="bool")

    @toggled.setter
    def toggled(self, value: Optional[bool]):
        self._set_attr("toggled", value)

    # max_value_length
    @property
    def max_value_length(self) -> OptionalNumber:
        return self._get_attr("maxValueLength")

    @max_value_length.setter
    def max_value_length(self, value: OptionalNumber):
        self._set_attr("maxValueLength", value)

    # checked
    @property
    def checked(self) -> Optional[bool]:
        return self._get_attr("checked", data_type="bool")

    @checked.setter
    def checked(self, value: Optional[bool]):
        self._set_attr("checked", value)

    # value
    @property
    def value(self) -> Optional[str]:
        return self._get_attr("value")

    @value.setter
    def value(self, value: Optional[str]):
        self._set_attr("value", value)

    # increased_value
    @property
    def increased_value(self) -> Optional[str]:
        return self._get_attr("increasedValue")

    @increased_value.setter
    def increased_value(self, value: Optional[str]):
        self._set_attr("increasedValue", value)

    # decreased_value
    @property
    def decreased_value(self) -> Optional[str]:
        return self._get_attr("decreasedValue")

    @decreased_value.setter
    def decreased_value(self, value: Optional[str]):
        self._set_attr("decreasedValue", value)

    # hint_text
    @property
    def hint_text(self) -> Optional[str]:
        return self._get_attr("hintText")

    @hint_text.setter
    def hint_text(self, value: Optional[str]):
        self._set_attr("hintText", value)

    # on_long_press_hint_text
    @property
    def on_long_press_hint_text(self):
        return self._get_attr("onLongPressHintText")

    @on_long_press_hint_text.setter
    def on_long_press_hint_text(self, value: Optional[bool]):
        self._set_attr("onLongPressHintText", value)

    # on_tap_hint_text
    @property
    def on_tap_hint_text(self):
        return self._get_attr("onTapHintText")

    @on_tap_hint_text.setter
    def on_tap_hint_text(self, value: Optional[bool]):
        self._set_attr("onTapHintText", value)

    # container
    @property
    def container(self) -> Optional[bool]:
        return self._get_attr("container", data_type="bool")

    @container.setter
    def container(self, value: Optional[bool]):
        self._set_attr("container", value)

    # live_region
    @property
    def live_region(self) -> Optional[bool]:
        return self._get_attr("liveRegion", data_type="bool")

    @live_region.setter
    def live_region(self, value: Optional[bool]):
        self._set_attr("liveRegion", value)

    # on_tap
    @property
    def on_tap(self) -> OptionalControlEventCallable:
        return self._get_event_handler("tap")

    @on_tap.setter
    def on_tap(self, handler: OptionalControlEventCallable):
        self._add_event_handler("tap", handler)
        self._set_attr("onTap", True if handler is not None else None)

    # on_double_tap
    @property
    def on_double_tap(self) -> OptionalControlEventCallable:
        return self._get_event_handler("double_tap")

    @on_double_tap.setter
    def on_double_tap(self, handler: OptionalControlEventCallable):
        self._add_event_handler("double_tap", handler)
        self._set_attr("onDoubleTap", True if handler is not None else None)

    # on_increase
    @property
    def on_increase(self) -> OptionalControlEventCallable:
        return self._get_event_handler("increase")

    @on_increase.setter
    def on_increase(self, handler: OptionalControlEventCallable):
        self._add_event_handler("increase", handler)
        self._set_attr("onIncrease", True if handler is not None else None)

    # on_decrease
    @property
    def on_decrease(self) -> OptionalControlEventCallable:
        return self._get_event_handler("decrease")

    @on_decrease.setter
    def on_decrease(self, handler: OptionalControlEventCallable):
        self._add_event_handler("decrease", handler)
        self._set_attr("onDecrease", True if handler is not None else None)

    # on_dismiss
    @property
    def on_dismiss(self) -> OptionalControlEventCallable:
        return self._get_event_handler("dismiss")

    @on_dismiss.setter
    def on_dismiss(self, handler: OptionalControlEventCallable):
        self._add_event_handler("dismiss", handler)
        self._set_attr("onDismiss", True if handler is not None else None)

    # on_scroll_left
    @property
    def on_scroll_left(self) -> OptionalControlEventCallable:
        return self._get_event_handler("scroll_left")

    @on_scroll_left.setter
    def on_scroll_left(self, handler: OptionalControlEventCallable):
        self._add_event_handler("scroll_left", handler)
        self._set_attr("onScrollLeft", True if handler is not None else None)

    # on_scroll_right
    @property
    def on_scroll_right(self) -> OptionalControlEventCallable:
        return self._get_event_handler("scroll_right")

    @on_scroll_right.setter
    def on_scroll_right(self, handler: OptionalControlEventCallable):
        self._add_event_handler("scroll_right", handler)
        self._set_attr("onScrollRight", True if handler is not None else None)

    # on_scroll_up
    @property
    def on_scroll_up(self) -> OptionalControlEventCallable:
        return self._get_event_handler("scroll_up")

    @on_scroll_up.setter
    def on_scroll_up(self, handler: OptionalControlEventCallable):
        self._add_event_handler("scroll_up", handler)
        self._set_attr("onScrollUp", True if handler is not None else None)

    # on_scroll_down
    @property
    def on_scroll_down(self) -> OptionalControlEventCallable:
        return self._get_event_handler("scroll_down")

    @on_scroll_down.setter
    def on_scroll_down(self, handler: OptionalControlEventCallable):
        self._add_event_handler("scroll_down", handler)
        self._set_attr("onScrollDown", True if handler is not None else None)

    # on_copy
    @property
    def on_copy(self) -> OptionalControlEventCallable:
        return self._get_event_handler("copy")

    @on_copy.setter
    def on_copy(self, handler: OptionalControlEventCallable):
        self._add_event_handler("copy", handler)
        self._set_attr("onCopy", True if handler is not None else None)

    # on_cut
    @property
    def on_cut(self) -> OptionalControlEventCallable:
        return self._get_event_handler("cut")

    @on_cut.setter
    def on_cut(self, handler: OptionalControlEventCallable):
        self._add_event_handler("cut", handler)
        self._set_attr("onCut", True if handler is not None else None)

    # on_paste
    @property
    def on_paste(self) -> OptionalControlEventCallable:
        return self._get_event_handler("paste")

    @on_paste.setter
    def on_paste(self, handler: OptionalControlEventCallable):
        self._add_event_handler("paste", handler)
        self._set_attr("onPaste", True if handler is not None else None)

    # on_long_press
    @property
    def on_long_press(self) -> OptionalControlEventCallable:
        return self._get_event_handler("long_press")

    @on_long_press.setter
    def on_long_press(self, handler: OptionalControlEventCallable):
        self._add_event_handler("long_press", handler)
        self._set_attr("onLongPress", True if handler is not None else None)

    # on_move_cursor_forward_by_character
    @property
    def on_move_cursor_forward_by_character(
        self,
    ) -> OptionalControlEventCallable:
        return self._get_event_handler("move_cursor_forward_by_character")

    @on_move_cursor_forward_by_character.setter
    def on_move_cursor_forward_by_character(
        self, handler: OptionalControlEventCallable
    ):
        self._add_event_handler("move_cursor_forward_by_character", handler)
        self._set_attr(
            "onMoveCursorForwardByCharacter", True if handler is not None else None
        )

    # on_move_cursor_backward_by_character
    @property
    def on_move_cursor_backward_by_character(
        self,
    ) -> OptionalControlEventCallable:
        return self._get_event_handler("move_cursor_backward_by_character")

    @on_move_cursor_backward_by_character.setter
    def on_move_cursor_backward_by_character(
        self, handler: OptionalControlEventCallable
    ):
        self._add_event_handler("move_cursor_backward_by_character", handler)
        self._set_attr(
            "onMoveCursorBackwardByCharacter", True if handler is not None else None
        )

    # on_did_gain_accessibility_focus
    @property
    def on_did_gain_accessibility_focus(
        self,
    ) -> OptionalControlEventCallable:
        return self._get_event_handler("did_gain_accessibility_focus")

    @on_did_gain_accessibility_focus.setter
    def on_did_gain_accessibility_focus(self, handler: OptionalControlEventCallable):
        self._add_event_handler("did_gain_accessibility_focus", handler)
        self._set_attr(
            "onDidGainAccessibilityFocus", True if handler is not None else None
        )

    # on_did_lose_accessibility_focus
    @property
    def on_did_lose_accessibility_focus(
        self,
    ) -> OptionalControlEventCallable:
        return self._get_event_handler("did_lose_accessibility_focus")

    @on_did_lose_accessibility_focus.setter
    def on_did_lose_accessibility_focus(self, handler: OptionalControlEventCallable):
        self._add_event_handler("did_lose_accessibility_focus", handler)
        self._set_attr(
            "onDidLoseAccessibilityFocus", True if handler is not None else None
        )

    # on_set_text
    @property
    def on_set_text(
        self,
    ) -> OptionalControlEventCallable:
        return self._get_event_handler("set_text")

    @on_set_text.setter
    def on_set_text(self, handler: OptionalControlEventCallable):
        self._add_event_handler("set_text", handler)
        self._set_attr("setText", True if handler is not None else None)



from typing import Any, Optional

from flet.core.control import Control
from flet.core.ref import Ref


class MapLayer(Control):
    """
    Abstract class for all map layers.
    """

    def __init__(
        self,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )



from typing import Any, Optional

from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.types import (
    ColorEnums,
    ColorValue,
    OptionalControlEventCallable,
    PaddingValue,
)


class CupertinoBottomSheet(Control):
    """
    A Cupertino version of modal bottom sheet.

    -----

    Online docs: https://flet.dev/docs/controls/cupertinobottomsheet
    """

    def __init__(
        self,
        content: Optional[Control] = None,
        open: bool = False,
        modal: bool = False,
        bgcolor: Optional[ColorValue] = None,
        height: OptionalNumber = None,
        padding: Optional[PaddingValue] = None,
        on_dismiss: OptionalControlEventCallable = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            disabled=disabled,
            visible=visible,
            data=data,
        )

        self.__content: Optional[Control] = None

        self.open = open
        self.modal = modal
        self.bgcolor = bgcolor
        self.height = height
        self.padding = padding
        self.content = content
        self.on_dismiss = on_dismiss

    def _get_control_name(self):
        return "cupertinobottomsheet"

    def before_update(self):
        super().before_update()
        self._set_attr_json("padding", self.__padding)

    def _get_children(self):
        children = []
        if self.__content:
            self.__content._set_attr_internal("n", "content")
            children.append(self.__content)
        return children

    # open
    @property
    def open(self) -> bool:
        return self._get_attr("open", data_type="bool", def_value=False)

    @open.setter
    def open(self, value: Optional[bool]):
        self._set_attr("open", value)

    # modal
    @property
    def modal(self) -> bool:
        return self._get_attr("modal", data_type="bool", def_value=False)

    @modal.setter
    def modal(self, value: Optional[bool]):
        self._set_attr("modal", value)

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgColor", value, ColorEnums)

    # height
    @property
    def height(self) -> float:
        return self._get_attr("height", data_type="float", def_value=220.0)

    @height.setter
    def height(self, value: OptionalNumber):
        self._set_attr("height", value)

    # padding
    @property
    def padding(self) -> Optional[PaddingValue]:
        return self.__padding

    @padding.setter
    def padding(self, value: Optional[PaddingValue]):
        self.__padding = value

    # content
    @property
    def content(self):
        return self.__content

    @content.setter
    def content(self, value):
        self.__content = value

    # on_dismiss
    @property
    def on_dismiss(self) -> OptionalControlEventCallable:
        return self._get_event_handler("dismiss")

    @on_dismiss.setter
    def on_dismiss(self, handler: OptionalControlEventCallable):
        self._add_event_handler("dismiss", handler)



from enum import Enum
from typing import Any, Optional, Union

from flet.core.adaptive_control import AdaptiveControl
from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.buttons import OutlinedBorder
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.text_style import TextStyle
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    MouseCursor,
    OffsetValue,
    OptionalControlEventCallable,
    PaddingValue,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
    UrlTarget,
    VisualDensity,
)


class ListTileTitleAlignment(Enum):
    TOP = "top"
    CENTER = "center"
    BOTTOM = "bottom"
    THREE_LINE = "threeLine"
    TITLE_HEIGHT = "titleHeight"


class ListTileStyle(Enum):
    LIST = "list"
    DRAWER = "drawer"


class ListTile(ConstrainedControl, AdaptiveControl):
    """
    A single fixed-height row that typically contains some text as well as a leading or trailing icon.

    Example:

    ```
    import flet as ft

    def main(page):
        page.title = "ListTile Example"
        page.add(
            ft.Card(
                content=ft.Container(
                    width=500,
                    content=ft.Column(
                        [
                            ft.ListTile(
                                title=ft.Text("One-line list tile"),
                            ),
                            ft.ListTile(
                                leading=ft.Icon(ft.icons.SETTINGS),
                                title=ft.Text("One-line selected list tile"),
                                selected=True,
                            ),
                        ],
                        spacing=0,
                    ),
                    padding=ft.padding.symmetric(vertical=10),
                )
            )
        )

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/listtile
    """

    def __init__(
        self,
        title: Optional[Control] = None,
        subtitle: Optional[Control] = None,
        is_three_line: Optional[bool] = None,
        leading: Optional[Control] = None,
        trailing: Optional[Control] = None,
        content_padding: PaddingValue = None,
        bgcolor: Optional[ColorValue] = None,
        bgcolor_activated: Optional[str] = None,
        hover_color: Optional[ColorValue] = None,
        selected: Optional[bool] = None,
        dense: Optional[bool] = None,
        autofocus: Optional[bool] = None,
        toggle_inputs: Optional[bool] = None,
        selected_color: Optional[ColorValue] = None,
        selected_tile_color: Optional[ColorValue] = None,
        style: Optional[ListTileStyle] = None,
        enable_feedback: Optional[bool] = None,
        horizontal_spacing: OptionalNumber = None,
        min_leading_width: OptionalNumber = None,
        min_vertical_padding: OptionalNumber = None,
        url: Optional[str] = None,
        url_target: Optional[UrlTarget] = None,
        title_alignment: Optional[ListTileTitleAlignment] = None,
        icon_color: Optional[ColorValue] = None,
        text_color: Optional[ColorValue] = None,
        shape: Optional[OutlinedBorder] = None,
        visual_density: Optional[VisualDensity] = None,
        mouse_cursor: Optional[MouseCursor] = None,
        title_text_style: Optional[TextStyle] = None,
        subtitle_text_style: Optional[TextStyle] = None,
        leading_and_trailing_text_style: Optional[TextStyle] = None,
        min_height: OptionalNumber = None,
        on_click: OptionalControlEventCallable = None,
        on_long_press: OptionalControlEventCallable = None,
        #
        # ConstrainedControl and AdaptiveControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
        adaptive: Optional[bool] = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        AdaptiveControl.__init__(self, adaptive=adaptive)

        self.content_padding = content_padding
        self.leading = leading
        self.title = title
        self.subtitle = subtitle
        self.trailing = trailing
        self.is_three_line = is_three_line
        self.selected = selected
        self.dense = dense
        self.autofocus = autofocus
        self.toggle_inputs = toggle_inputs
        self.url = url
        self.url_target = url_target
        self.bgcolor = bgcolor
        self.bgcolor_activated = bgcolor_activated
        self.hover_color = hover_color
        self.on_click = on_click
        self.on_long_press = on_long_press
        self.style = style
        self.selected_color = selected_color
        self.selected_tile_color = selected_tile_color
        self.enable_feedback = enable_feedback
        self.horizontal_spacing = horizontal_spacing
        self.min_leading_width = min_leading_width
        self.min_vertical_padding = min_vertical_padding
        self.title_alignment = title_alignment
        self.icon_color = icon_color
        self.text_color = text_color
        self.shape = shape
        self.visual_density = visual_density
        self.mouse_cursor = mouse_cursor
        self.title_text_style = title_text_style
        self.subtitle_text_style = subtitle_text_style
        self.leading_and_trailing_text_style = leading_and_trailing_text_style
        self.min_height = min_height

    def _get_control_name(self):
        return "listtile"

    def before_update(self):
        super().before_update()
        self._set_attr_json("contentPadding", self.__content_padding)
        if isinstance(self.__shape, OutlinedBorder):
            self._set_attr_json("shape", self.__shape)
        if isinstance(self.__title_text_style, TextStyle):
            self._set_attr_json("titleTextStyle", self.__title_text_style)
        if isinstance(self.__subtitle_text_style, TextStyle):
            self._set_attr_json("subtitleTextStyle", self.__subtitle_text_style)
        if isinstance(self.__leading_and_trailing_text_style, TextStyle):
            self._set_attr_json(
                "leadingAndTrailingTextStyle", self.__leading_and_trailing_text_style
            )

    def _get_children(self):
        children = []
        if self.__leading:
            self.__leading._set_attr_internal("n", "leading")
            children.append(self.__leading)
        if self.__title:
            self.__title._set_attr_internal("n", "title")
            children.append(self.__title)
        if self.__subtitle:
            self.__subtitle._set_attr_internal("n", "subtitle")
            children.append(self.__subtitle)
        if self.__trailing:
            self.__trailing._set_attr_internal("n", "trailing")
            children.append(self.__trailing)
        return children

    # content_padding
    @property
    def content_padding(self) -> Optional[PaddingValue]:
        return self.__content_padding

    @content_padding.setter
    def content_padding(self, value: Optional[PaddingValue]):
        self.__content_padding = value

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # selected_color
    @property
    def selected_color(self) -> Optional[ColorValue]:
        return self.__selected_color

    @selected_color.setter
    def selected_color(self, value: Optional[ColorValue]):
        self.__selected_color = value
        self._set_enum_attr("selectedColor", value, ColorEnums)

    # selected_tile_color
    @property
    def selected_tile_color(self) -> Optional[ColorValue]:
        return self.__selected_tile_color

    @selected_tile_color.setter
    def selected_tile_color(self, value: Optional[ColorValue]):
        self.__selected_tile_color = value
        self._set_enum_attr("selectedTileColor", value, ColorEnums)

    # bgcolor_activated
    @property
    def bgcolor_activated(self) -> Optional[str]:
        return self._get_attr("bgcolorActivated")

    @bgcolor_activated.setter
    def bgcolor_activated(self, value: Optional[str]):
        self._set_attr("bgcolorActivated", value)

    # min_leading_width
    @property
    def min_leading_width(self) -> float:
        return self._get_attr("minLeadingWidth", data_type="float", def_value=40.0)

    @min_leading_width.setter
    def min_leading_width(self, value: OptionalNumber):
        self._set_attr("minLeadingWidth", value)

    # horizontal_spacing
    @property
    def horizontal_spacing(self) -> float:
        return self._get_attr("horizontalSpacing", data_type="float", def_value=16.0)

    @horizontal_spacing.setter
    def horizontal_spacing(self, value: OptionalNumber):
        self._set_attr("horizontalSpacing", value)

    # min_height
    @property
    def min_height(self) -> OptionalNumber:
        return self._get_attr("minHeight", data_type="float")

    @min_height.setter
    def min_height(self, value: OptionalNumber):
        self._set_attr("minHeight", value)

    # hover_color
    @property
    def hover_color(self) -> Optional[ColorValue]:
        return self.__hover_color

    @hover_color.setter
    def hover_color(self, value: Optional[ColorValue]):
        self.__hover_color = value
        self._set_enum_attr("hoverColor", value, ColorEnums)

    # leading
    @property
    def leading(self) -> Optional[Control]:
        return self.__leading

    @leading.setter
    def leading(self, value: Optional[Control]):
        self.__leading = value

    # title
    @property
    def title(self) -> Optional[Control]:
        return self.__title

    @title.setter
    def title(self, value: Optional[Control]):
        self.__title = value

    # min_vertical_padding
    @property
    def min_vertical_padding(self) -> float:
        return self._get_attr("minVerticalPadding", data_type="float", def_value=4.0)

    @min_vertical_padding.setter
    def min_vertical_padding(self, value: OptionalNumber):
        self._set_attr("minVerticalPadding", value)

    # subtitle
    @property
    def subtitle(self) -> Optional[Control]:
        return self.__subtitle

    @subtitle.setter
    def subtitle(self, value: Optional[Control]):
        self.__subtitle = value

    # trailing
    @property
    def trailing(self) -> Optional[Control]:
        return self.__trailing

    @trailing.setter
    def trailing(self, value: Optional[Control]):
        self.__trailing = value

    # is_three_line
    @property
    def is_three_line(self) -> bool:
        return self._get_attr("isThreeLine", data_type="bool", def_value=False)

    @is_three_line.setter
    def is_three_line(self, value: Optional[bool]):
        self._set_attr("isThreeLine", value)

    # enable_feedback
    @property
    def enable_feedback(self) -> bool:
        return self._get_attr("enableFeedback", data_type="bool", def_value=True)

    @enable_feedback.setter
    def enable_feedback(self, value: Optional[bool]):
        self._set_attr("enableFeedback", value)

    # style
    @property
    def style(self) -> Optional[ListTileStyle]:
        return self.__style

    @style.setter
    def style(self, value: Optional[ListTileStyle]):
        self.__style = value
        self._set_enum_attr("style", value, ListTileStyle)

    # title_alignment
    @property
    def title_alignment(self) -> Optional[ListTileTitleAlignment]:
        return self.__title_alignment

    @title_alignment.setter
    def title_alignment(self, value: Optional[ListTileTitleAlignment]):
        self.__title_alignment = value
        self._set_enum_attr("titleAlignment", value, ListTileTitleAlignment)

    # selected
    @property
    def selected(self) -> bool:
        return self._get_attr("selected", data_type="bool", def_value=False)

    @selected.setter
    def selected(self, value: Optional[bool]):
        self._set_attr("selected", value)

    # dense
    @property
    def dense(self) -> bool:
        return self._get_attr("dense", data_type="bool", def_value=False)

    @dense.setter
    def dense(self, value: Optional[bool]):
        self._set_attr("dense", value)

    # autofocus
    @property
    def autofocus(self) -> bool:
        return self._get_attr("autofocus", data_type="bool", def_value=False)

    @autofocus.setter
    def autofocus(self, value: Optional[bool]):
        self._set_attr("autofocus", value)

    # toggle_inputs
    @property
    def toggle_inputs(self) -> bool:
        return self._get_attr("toggleInputs", data_type="bool", def_value=False)

    @toggle_inputs.setter
    def toggle_inputs(self, value: Optional[bool]):
        self._set_attr("toggleInputs", value)

    # url
    @property
    def url(self) -> Optional[str]:
        return self._get_attr("url")

    @url.setter
    def url(self, value: Optional[str]):
        self._set_attr("url", value)

    # icon_color
    @property
    def icon_color(self) -> Optional[ColorValue]:
        return self.__icon_color

    @icon_color.setter
    def icon_color(self, value: Optional[ColorValue]):
        self.__icon_color = value
        self._set_enum_attr("iconColor", value, ColorEnums)

    # text_color
    @property
    def text_color(self) -> Optional[ColorValue]:
        return self.__text_color

    @text_color.setter
    def text_color(self, value: Optional[ColorValue]):
        self.__text_color = value
        self._set_enum_attr("textColor", value, ColorEnums)

    # url_target
    @property
    def url_target(self) -> Optional[UrlTarget]:
        return self.__url_target

    @url_target.setter
    def url_target(self, value: Optional[UrlTarget]):
        self.__url_target = value
        self._set_enum_attr("urlTarget", value, UrlTarget)

    # mouse_cursor
    @property
    def mouse_cursor(self) -> Optional[MouseCursor]:
        return self.__mouse_cursor

    @mouse_cursor.setter
    def mouse_cursor(self, value: Optional[MouseCursor]):
        self.__mouse_cursor = value
        self._set_enum_attr("mouseCursor", value, MouseCursor)

    # visual_density
    @property
    def visual_density(self) -> Optional[VisualDensity]:
        return self.__visual_density

    @visual_density.setter
    def visual_density(self, value: Optional[VisualDensity]):
        self.__visual_density = value
        self._set_enum_attr("visualDensity", value, VisualDensity)

    # shape
    @property
    def shape(self) -> Optional[OutlinedBorder]:
        return self.__shape

    @shape.setter
    def shape(self, value: Optional[OutlinedBorder]):
        self.__shape = value

    # title_text_style
    @property
    def title_text_style(self) -> Optional[TextStyle]:
        return self.__title_text_style

    @title_text_style.setter
    def title_text_style(self, value: Optional[TextStyle]):
        self.__title_text_style = value

    # subtitle_text_style
    @property
    def subtitle_text_style(self) -> Optional[TextStyle]:
        return self.__subtitle_text_style

    @subtitle_text_style.setter
    def subtitle_text_style(self, value: Optional[TextStyle]):
        self.__subtitle_text_style = value

    # leading_and_trailing_text_style
    @property
    def leading_and_trailing_text_style(self) -> Optional[TextStyle]:
        return self.__leading_and_trailing_text_style

    @leading_and_trailing_text_style.setter
    def leading_and_trailing_text_style(self, value: Optional[TextStyle]):
        self.__leading_and_trailing_text_style = value

    # on_click
    @property
    def on_click(self) -> OptionalControlEventCallable:
        return self._get_event_handler("click")

    @on_click.setter
    def on_click(self, handler: OptionalControlEventCallable):
        self._add_event_handler("click", handler)
        self._set_attr("onclick", True if handler is not None else None)

    # on_long_press
    @property
    def on_long_press(self) -> OptionalControlEventCallable:
        return self._get_event_handler("long_press")

    @on_long_press.setter
    def on_long_press(self, handler: OptionalControlEventCallable):
        self._add_event_handler("long_press", handler)
        self._set_attr("onLongPress", True if handler is not None else None)

    # on_focus
    @property
    def on_focus(self) -> OptionalControlEventCallable:
        return self._get_event_handler("focus")

    @on_focus.setter
    def on_focus(self, handler: OptionalControlEventCallable):
        self._add_event_handler("focus", handler)

    # on_blur
    @property
    def on_blur(self) -> OptionalControlEventCallable:
        return self._get_event_handler("blur")

    @on_blur.setter
    def on_blur(self, handler: OptionalControlEventCallable):
        self._add_event_handler("blur", handler)



from typing import Optional

from flet.core.control import Control


class AdaptiveControl(Control):
    def __init__(self, adaptive: Optional[bool] = None):
        self.adaptive = adaptive

    # adaptive
    @property
    def adaptive(self) -> bool:
        return self._get_attr("adaptive", data_type="bool", def_value=False)

    @adaptive.setter
    def adaptive(self, value: Optional[bool]):
        self._set_attr("adaptive", value)



from typing import Any, Optional

from flet.core.control import Control
from flet.core.ref import Ref
from flet.core.text_style import TextStyle
from flet.core.types import OptionalControlEventCallable


class TextSourceAttribution(Control):
    """
    A text source attribution displayed on the Map.
    For it to be displayed, it should be part of a RichAttribution.attributions list.

    -----

    Online docs: https://flet.dev/docs/controls/maptextsourceattribution
    """

    def __init__(
        self,
        text: str,
        text_style: Optional[TextStyle] = None,
        prepend_copyright: Optional[bool] = None,
        on_click: OptionalControlEventCallable = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):

        Control.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )

        self.text = text
        self.text_style = text_style
        self.prepend_copyright = prepend_copyright
        self.on_click = on_click

    def _get_control_name(self):
        return "map_text_source_attribution"

    def before_update(self):
        super().before_update()
        if isinstance(self.__text_style, TextStyle):
            self._set_attr_json("textStyle", self.__text_style)

    # text_style
    @property
    def text_style(self) -> Optional[TextStyle]:
        return self.__text_style

    @text_style.setter
    def text_style(self, value: Optional[TextStyle]):
        self.__text_style = value

    # prepend_copyright
    @property
    def prepend_copyright(self) -> bool:
        return self._get_attr("prependCopyright", data_type="bool", def_value=True)

    @prepend_copyright.setter
    def prepend_copyright(self, value: Optional[bool]):
        self._set_attr("prependCopyright", value)

    # text
    @property
    def text(self) -> str:
        return self._get_attr("text")

    @text.setter
    def text(self, value: str):
        self._set_attr("text", value)

    # on_click
    @property
    def on_click(self) -> OptionalControlEventCallable:
        return self._get_event_handler("click")

    @on_click.setter
    def on_click(self, handler: OptionalControlEventCallable):
        self._add_event_handler("click", handler)



from typing import Any, List, Optional

from flet.core.alignment import Alignment
from flet.core.control import Control, OptionalNumber
from flet.core.map.map import MapLatitudeLongitude
from flet.core.map.map_layer import MapLayer
from flet.core.ref import Ref


class Marker(Control):
    """
    A marker displayed on the Map at the specified location through the MarkerLayer.

    -----

    Online docs: https://flet.dev/docs/controls/mapmarker
    """

    def __init__(
        self,
        content: Control,
        coordinates: MapLatitudeLongitude,
        rotate: Optional[bool] = None,
        height: OptionalNumber = None,
        width: OptionalNumber = None,
        alignment: Optional[Alignment] = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):
        Control.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )

        self.content = content
        self.coordinates = coordinates
        self.rotate = rotate
        self.height = height
        self.width = width
        self.alignment = alignment

    def _get_control_name(self):
        return "map_marker"

    def _get_children(self):
        return [self.__content]

    def before_update(self):
        super().before_update()
        self._set_attr_json("alignment", self.__alignment)
        self._set_attr_json("coordinates", self.__coordinates)

    # content
    @property
    def content(self) -> Optional[Alignment]:
        return self.__content

    @content.setter
    def content(self, value: Optional[Alignment]):
        self.__content = value

    # rotate
    @property
    def rotate(self) -> bool:
        return self._get_attr("rotate", data_type="bool", def_value=False)

    @rotate.setter
    def rotate(self, value: Optional[bool]):
        self._set_attr("rotate", value)

    # height
    @property
    def height(self) -> float:
        return self._get_attr("height", data_type="float", def_value=30.0)

    @height.setter
    def height(self, value: OptionalNumber):
        assert value is None or value >= 0, "height cannot be negative"
        self._set_attr("height", value)

    # width
    @property
    def width(self) -> float:
        return self._get_attr("width", data_type="float", def_value=30.0)

    @width.setter
    def width(self, value: OptionalNumber):
        assert value is None or value >= 0, "width cannot be negative"
        self._set_attr("width", value)

    # alignment
    @property
    def alignment(self) -> Optional[Alignment]:
        return self.__alignment

    @alignment.setter
    def alignment(self, value: Optional[Alignment]):
        self.__alignment = value

    # coordinates
    @property
    def coordinates(self) -> MapLatitudeLongitude:
        return self.__coordinates

    @coordinates.setter
    def coordinates(self, value: MapLatitudeLongitude):
        self.__coordinates = value


class MarkerLayer(MapLayer):
    """
    A layer to display Markers.

    -----

    Online docs: https://flet.dev/docs/controls/mapmarkerlayer
    """

    def __init__(
        self,
        markers: List[Marker],
        alignment: Optional[Alignment] = None,
        rotate: Optional[bool] = None,
        #
        # MapLayer
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):
        MapLayer.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )

        self.markers = markers
        self.alignment = alignment
        self.rotate = rotate

    def _get_control_name(self):
        return "map_marker_layer"

    def _get_children(self):
        return self.__markers

    def before_update(self):
        super().before_update()
        self._set_attr_json("alignment", self.__alignment)

    # alignment
    @property
    def alignment(self) -> Optional[Alignment]:
        return self.__alignment

    @alignment.setter
    def alignment(self, value: Optional[Alignment]):
        self.__alignment = value

    # markers
    @property
    def markers(self) -> List[Marker]:
        return self.__markers

    @markers.setter
    def markers(self, value: List[Marker]):
        self.__markers = value

    # rotate
    @property
    def rotate(self) -> bool:
        return self._get_attr("rotate", data_type="bool", def_value=False)

    @rotate.setter
    def rotate(self, value: Optional[bool]):
        self._set_attr("rotate", value)



import logging

import flet.core
from flet.core.connection import Connection
from flet.core.protocol import *

logger = logging.getLogger(flet.__name__)


class LocalConnection(Connection):
    def __init__(self):
        super().__init__()
        self.__control_id = 1
        self._client_details = None

    def _create_register_web_client_response(
        self, controls: Optional[Dict[str, Dict[str, Any]]] = None
    ):
        assert self._client_details
        return ClientMessage(
            ClientActions.REGISTER_WEB_CLIENT,
            RegisterWebClientResponsePayload(
                session=SessionPayload(
                    id=self._client_details.sessionId,
                    controls=(
                        controls
                        if controls is not None
                        else {
                            "page": {
                                "i": "page",
                                "t": "page",
                                "p": "",
                                "c": [],
                                "route": self._client_details.pageRoute,
                                "width": self._client_details.pageWidth,
                                "height": self._client_details.pageHeight,
                                "windowwidth": self._client_details.windowWidth,
                                "windowheight": self._client_details.windowHeight,
                                "windowtop": self._client_details.windowTop,
                                "windowleft": self._client_details.windowLeft,
                                "pwa": self._client_details.isPWA,
                                "web": self._client_details.isWeb,
                                "debug": self._client_details.isDebug,
                                "platform": self._client_details.platform,
                                "platformBrightness": self._client_details.platformBrightness,
                                "media": self._client_details.media,
                            }
                        }
                    ),
                ),
                appInactive=False,
                error="",
            ),
        )

    def _create_session_handler_arg(self):
        assert self._client_details
        return PageSessionCreatedPayload(
            pageName=self._client_details.pageName,
            sessionID=self._client_details.sessionId,
        )

    def _create_page_event_handler_arg(self, msg: ClientMessage):
        assert self._client_details
        web_event = PageEventFromWebPayload(**msg.payload)
        return PageEventPayload(
            pageName=self._client_details.pageName,
            sessionID=self._client_details.sessionId,
            eventTarget=web_event.eventTarget,
            eventName=web_event.eventName,
            eventData=web_event.eventData,
        )

    def _create_update_control_props_handler_arg(self, msg: ClientMessage):
        assert self._client_details
        return PageEventPayload(
            pageName=self._client_details.pageName,
            sessionID=self._client_details.sessionId,
            eventTarget="page",
            eventName="change",
            eventData=json.dumps(msg.payload["props"], separators=(",", ":")),
        )

    def _process_command(self, command: Command):
        logger.debug(f"_process_command: {command}")
        if command.name == "get":
            return self._process_get_command(command.values)
        elif command.name == "add":
            return self._process_add_command(command)
        elif command.name == "set":
            return self._process_set_command(command.values, command.attrs)
        elif command.name == "remove":
            return self._process_remove_command(command.values)
        elif command.name == "clean":
            return self._process_clean_command(command.values)
        elif command.name == "invokeMethod":
            return self._process_invoke_method_command(command.values, command.attrs)
        elif command.name == "error":
            return self._process_error_command(command.values)
        elif command.name == "getUploadUrl":
            return self._process_get_upload_url_command(command.attrs)
        elif command.name == "oauthAuthorize":
            return self._process_oauth_authorize_command(command.attrs)
        raise Exception(f"Unsupported command: {command.name}")

    def _process_add_command(self, command: Command):
        top_parent_id = command.attrs.get("to", "page")
        top_parent_at = int(command.attrs.get("at", "-1"))

        batch: List[Command] = []
        if len(command.values) > 0:
            batch.append(command)

        for sub_cmd in command.commands:
            sub_cmd.name = "add"
            batch.append(sub_cmd)

        ids = []
        controls = []
        controls_idx = {}

        i = 0
        for cmd in batch:
            assert len(cmd.values) > 0, "control type is not specified"
            control_type = cmd.values[0].lower()

            parent_id = ""
            parent_at = -1

            # find nearest parentID
            pi = i - 1
            while pi >= 0:
                if batch[pi].indent < cmd.indent:
                    parent_id = batch[pi].attrs.get("id", "")
                    break
                pi -= 1

            # parent wasn't found - use the topmost one
            if parent_id == "":
                parent_id = top_parent_id
                parent_at = top_parent_at

            id = cmd.attrs.get("id", "")
            if not id:
                id = f"_{self._get_next_control_id()}"
                cmd.attrs["id"] = id

            ids.append(id)

            control = {"t": control_type, "i": id, "p": parent_id, "c": []}
            controls.append(control)
            controls_idx[id] = control

            if parent_at != -1:
                control["at"] = str(parent_at)
                top_parent_at += 1

            parent_control = controls_idx.get(parent_id)
            if parent_control:
                if parent_at != -1:
                    parent_control["c"].insert(parent_at, id)
                else:
                    parent_control["c"].append(id)

            system_attrs = ["id", "to", "from", "at", "t", "p", "i", "c"]
            for k, v in cmd.attrs.items():
                if k not in system_attrs and v:
                    control[k] = v

            i += 1

        return " ".join(ids), ClientMessage(
            ClientActions.ADD_PAGE_CONTROLS, AddPageControlsPayload(controls=controls)
        )

    def _process_set_command(self, values, attrs):
        assert len(values) == 1, '"set" command has wrong number of values'
        props = {"i": values[0]}
        for k, v in attrs.items():
            props[k] = v

        return "", ClientMessage(
            ClientActions.UPDATE_CONTROL_PROPS, UpdateControlPropsPayload(props=[props])
        )

    def _process_remove_command(self, values):
        assert len(values) > 0, '"remove" command has wrong number of values'
        return "", ClientMessage(
            ClientActions.REMOVE_CONTROL, RemoveControlPayload(ids=values)
        )

    def _process_clean_command(self, values):
        assert len(values) > 0, '"clean" command has wrong number of values'
        return "", ClientMessage(
            ClientActions.CLEAN_CONTROL, CleanControlPayload(ids=values)
        )

    def _process_error_command(self, values):
        assert len(values) == 1, '"error" command has wrong number of values'
        return "", ClientMessage(
            ClientActions.SESSION_CRASHED, SessionCrashedPayload(message=values[0])
        )

    def _process_invoke_method_command(self, values, attrs):
        # "invokeMethod", values=[method_id, method_name], attrs=arguments
        assert len(values) == 3, '"invokeMethod" command has wrong number of values'
        return "", ClientMessage(
            ClientActions.INVOKE_METHOD,
            InvokeMethodPayload(
                methodId=values[0],
                methodName=values[1],
                controlId=values[2],
                arguments=attrs,
            ),
        )

    def _process_get_upload_url_command(self, attrs):
        raise Exception("getUploadUrl command is not supported.")

    def _process_oauth_authorize_command(self, attrs):
        raise Exception("oauthAuthorize command is not supported.")

    def _process_get_command(self, values: List[str]):
        assert len(values) == 2, '"get" command has wrong number of values'
        assert self._client_details
        ctrl_id = values[0]
        prop_name = values[1]
        r = ""
        if ctrl_id == "page":
            if prop_name == "route":
                r = self._client_details.pageRoute
            elif prop_name == "pwa":
                r = self._client_details.isPWA
            elif prop_name == "web":
                r = self._client_details.isWeb
            elif prop_name == "debug":
                r = self._client_details.isDebug
            elif prop_name == "platform":
                r = self._client_details.platform
            elif prop_name == "platformBrightness":
                r = self._client_details.platformBrightness
            elif prop_name == "media":
                r = self._client_details.media
            elif prop_name == "width":
                r = self._client_details.pageWidth
            elif prop_name == "height":
                r = self._client_details.pageHeight
            elif prop_name == "windowWidth":
                r = self._client_details.windowWidth
            elif prop_name == "windowHeight":
                r = self._client_details.windowHeight
            elif prop_name == "windowTop":
                r = self._client_details.windowTop
            elif prop_name == "windowLeft":
                r = self._client_details.windowLeft
        return r, None

    def _get_next_control_id(self):
        r = self.__control_id
        self.__control_id += 1
        return r



from flet.core.map.circle_layer import CircleLayer, CircleMarker
from flet.core.map.map import (
    Map,
    MapEvent,
    MapEventSource,
    MapHoverEvent,
    MapInteractionConfiguration,
    MapInteractiveFlag,
    MapLatitudeLongitude,
    MapLatitudeLongitudeBounds,
    MapMultiFingerGesture,
    MapPointerDeviceType,
    MapPointerEvent,
    MapPositionChangeEvent,
    MapTapEvent,
)
from flet.core.map.marker_layer import Marker, MarkerLayer
from flet.core.map.polygon_layer import PolygonLayer, PolygonMarker
from flet.core.map.polyline_layer import (
    DashedStrokePattern,
    DottedStrokePattern,
    PatternFit,
    PolylineLayer,
    PolylineMarker,
    SolidStrokePattern,
)
from flet.core.map.rich_attribution import RichAttribution
from flet.core.map.simple_attribution import SimpleAttribution
from flet.core.map.text_source_attribution import TextSourceAttribution
from flet.core.map.tile_layer import MapTileLayerEvictErrorTileStrategy, TileLayer



import subprocess
import sys

import flet.version


def install_flet_package(name: str):
    print(f"Installing {name} {flet.version.version} package...", end="")
    retcode = subprocess.call(
        [
            sys.executable,
            "-m",
            "pip",
            "install",
            "-q",
            "--disable-pip-version-check",
            f"{name}=={flet.version.version}",
        ]
    )
    if retcode == 0:
        print("OK")
    else:
        print(
            f'Unable to upgrade "{name}" package to version {flet.version.version}. Please use "pip install \'flet[all]=={flet.version.version}\' --upgrade" command to upgrade Flet.'
        )
        exit(1)


def ensure_flet_desktop_package_installed():
    try:
        import flet.version
        import flet_desktop.version

        assert (
            not flet_desktop.version.version
            or flet_desktop.version.version == flet.version.version
        )
    except:
        install_flet_package("flet-desktop")


def ensure_flet_web_package_installed():
    try:
        import flet.version
        import flet_web.version

        assert (
            not flet_web.version.version
            or flet_web.version.version == flet.version.version
        )
    except:
        install_flet_package("flet-web")


def ensure_flet_cli_package_installed():
    try:
        import flet.version
        import flet_cli.version

        assert (
            not flet_cli.version.version
            or flet_cli.version.version == flet.version.version
        )
    except:
        install_flet_package("flet-cli")



from dataclasses import dataclass
from enum import Enum, IntFlag
from typing import List, Optional, Union

from flet.core.box import BoxShadow
from flet.core.painting import Paint
from flet.core.types import ColorValue, FontWeight, OptionalNumber


class TextOverflow(Enum):
    CLIP = "clip"
    ELLIPSIS = "ellipsis"
    FADE = "fade"
    VISIBLE = "visible"


class TextBaseline(Enum):
    ALPHABETIC = "alphabetic"
    IDEOGRAPHIC = "ideographic"


class TextThemeStyle(Enum):
    DISPLAY_LARGE = "displayLarge"
    DISPLAY_MEDIUM = "displayMedium"
    DISPLAY_SMALL = "displaySmall"
    HEADLINE_LARGE = "headlineLarge"
    HEADLINE_MEDIUM = "headlineMedium"
    HEADLINE_SMALL = "headlineSmall"
    TITLE_LARGE = "titleLarge"
    TITLE_MEDIUM = "titleMedium"
    TITLE_SMALL = "titleSmall"
    LABEL_LARGE = "labelLarge"
    LABEL_MEDIUM = "labelMedium"
    LABEL_SMALL = "labelSmall"
    BODY_LARGE = "bodyLarge"
    BODY_MEDIUM = "bodyMedium"
    BODY_SMALL = "bodySmall"


class TextDecoration(IntFlag):
    NONE = 0
    UNDERLINE = 1
    OVERLINE = 2
    LINE_THROUGH = 4


class TextDecorationStyle(Enum):
    SOLID = "solid"
    DOUBLE = "double"
    DOTTED = "dotted"
    DASHED = "dashed"
    WAVY = "wavy"


@dataclass
class TextStyle:
    size: OptionalNumber = None
    height: OptionalNumber = None
    weight: Optional[FontWeight] = None
    italic: Optional[bool] = None
    decoration: Optional[TextDecoration] = None
    decoration_color: Optional[ColorValue] = None
    decoration_thickness: OptionalNumber = None
    decoration_style: Optional[TextDecorationStyle] = None
    font_family: Optional[str] = None
    color: Optional[ColorValue] = None
    bgcolor: Optional[ColorValue] = None
    shadow: Union[None, BoxShadow, List[BoxShadow]] = None
    foreground: Optional[Paint] = None
    letter_spacing: OptionalNumber = None
    word_spacing: OptionalNumber = None
    overflow: Optional[TextOverflow] = None
    baseline: Optional[TextBaseline] = None


@dataclass
class StrutStyle:
    size: OptionalNumber = None
    height: OptionalNumber = None
    weight: Optional[FontWeight] = None
    italic: Optional[bool] = None
    font_family: Optional[str] = None
    leading: OptionalNumber = None
    force_strut_height: Optional[bool] = None



from flet.core.elevated_button import ElevatedButton


class FilledButton(ElevatedButton):
    """
    Filled buttons have the most visual impact after the FloatingActionButton (https://flet.dev/docs/controls/floatingactionbutton), and should be used for important, final actions that complete a flow, like Save, Join now, or Confirm.

    Example:
    ```
    import flet as ft


    def main(page: ft.Page):
        page.title = "Basic filled buttons"
        page.add(
            ft.FilledButton(text="Filled button"),
            ft.FilledButton("Disabled button", disabled=True),
            ft.FilledButton("Button with icon", icon="add"),
        )

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/filledbutton
    """

    def _get_control_name(self):
        return "filledbutton"



import enum
import json
from typing import Dict

from flet.core.border import Border, BorderSide
from flet.core.border_radius import BorderRadius
from flet.core.box import BoxConstraints
from flet.core.margin import Margin
from flet.core.padding import Padding


class EmbedJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, BorderSide):
            obj_as_dict = {
                "w": obj.width,
                "c": obj.color,
                "sa": obj.stroke_align,
            }
        elif isinstance(obj, Border):
            obj_as_dict = {
                "l": obj.left,
                "t": obj.top,
                "r": obj.right,
                "b": obj.bottom,
            }
        elif isinstance(obj, BorderRadius):
            obj_as_dict = {
                "bl": obj.bottom_left,
                "br": obj.bottom_right,
                "tl": obj.top_left,
                "tr": obj.top_right,
            }
        elif isinstance(obj, (Margin, Padding)):
            obj_as_dict = {
                "l": obj.left,
                "t": obj.top,
                "r": obj.right,
                "b": obj.bottom,
            }
        elif isinstance(obj, BoxConstraints):
            obj_as_dict = {
                "min_width": obj.min_width,
                "max_width": obj.max_width,
                "min_height": obj.min_height,
                "max_height": obj.max_height,
            }
        else:
            obj_as_dict = self._convert_enums(obj.__dict__)

        # Convert inf to string "inf" to avoid JSON serialization error
        for key, value in obj_as_dict.items():
            if value == float("inf"):
                obj_as_dict[key] = "inf"

        return obj_as_dict

    def encode(self, o):
        return super().encode(self._convert_enums(o))

    def _convert_enums(self, obj):
        if isinstance(obj, Dict):
            return dict(
                map(
                    lambda item: (
                        self._convert_enums(
                            item[0]
                            if not isinstance(item[0], enum.Enum)
                            else item[0].value
                        ),
                        self._convert_enums(
                            item[1]
                            if not isinstance(item[1], enum.Enum)
                            else item[1].value
                        ),
                    ),
                    filter(lambda item: item[1] is not None, obj.items()),
                )
            )
        else:
            return obj
